/**
 *  Class ChatDialog.java
 *  @author: Anders Sandstrom
 *  UCCS 1997
 */

//ChatDialog.java
import java.awt.*;
import java.awt.event.*;
import java.util.*;
public class ChatDialog extends Dialog implements ActionListener {

  // Variable declarations
  private String userName="Unconnected";
  private TextArea chatArea;
  private TextField input;
  private Button sendButton;
  private Button closeButton;
  private ResultProcessor owner ;
  private Vector names;
  private Frame parent;
  private Color colors[]={Color.black,Color.blue,Color.cyan,
                  Color.green,Color.magenta,Color.orange,
                  Color.pink,Color.red,Color.yellow};

  // Constructor
  public ChatDialog(Frame parent,String title,boolean modal,ResultProcessor owner) {
	super(parent, title, modal);
	this.parent=parent;
	this.owner=owner;
	//userName=name;
    setup();
  }

  // Method setup()
	void setup(){
    names=new Vector();
    names.addElement(userName);
    setupGUI();
  }

  // Method show()
  public void setVisible(boolean b) {
    if(b){
      Rectangle bounds = parent.getBounds();
      Rectangle abounds = getBounds();
      setLocation(bounds.x + (bounds.width - abounds.width)/ 2,
    	   bounds.y + (bounds.height - abounds.height)/2);
      super.setVisible(true);
    }
    else super.setVisible(false);
  }

  // Method wakeUp()
  public synchronized void wakeUp() {
    notify();
  }

  // Method setUserName()
  public void setUserName(String name){
    userName=name;
  }

  // Method action()
  public void actionPerformed(ActionEvent event){
    String cmd=event.getActionCommand();
    System.out.println("cmd: "+cmd);

    if(cmd.equals("close")){
      dispose();
      setVisible(false);
    }
    else handleInput();
  }

  // Method handleInput()
  void handleInput(){
    String msg=input.getText();
    System.out.println("msg: ."+msg+".");
    input.setText("");
    input.requestFocus();
    if(msg!=""){
      printToLocal(msg);
      owner.sendChatData(encodeMessage(msg));
    }
  }

  // Method printToLocal()
  void printToLocal(String msg){
    System.out.println("Caret: "+chatArea.getCaretPosition());
    String string=new String(userName+": "+msg+"\n");
    //chatArea.setForeground(colors[getColorIndex(userName)]);
    //chatArea.get
    chatArea.append(string);
    //chatArea.insert(string,chatArea.getCaretPosition());
  }

  // Method printRemote()
  public void printRemote(String name,String msg){
    String string=new String(name+":"+decodeMessage(msg)+"\n");
    //chatArea.setForeground(colors[getColorIndex(name)]);
    chatArea.append(string);
    //chatArea.insert(string,chatArea.getCaretPosition());
  }

  // Method encodeMessage()
  String encodeMessage(String msg){
    StringTokenizer encoder=new StringTokenizer(msg);
    String temp="";
    while(encoder.hasMoreTokens()) {
      temp=new String(temp+"|"+encoder.nextToken());
    }
    return temp;
  }

  // Method decodeMessage()
  String decodeMessage(String msg){
    StringTokenizer decoder=new StringTokenizer(msg,"|");
    String temp="";
    while(decoder.hasMoreTokens()) {
      temp=new String(temp+" "+decoder.nextToken());
    }
    return temp;
  }

  // Method getColorIndex()
    int getColorIndex(String name){
    int index=0;
    for(int i=0;i<names.size();i++){
      if(((String)names.elementAt(i))==name) index=i;
    }
    if(index==0){
      names.addElement(name);
      index=names.size()-1;
    }
    return index;
  }




  // setupGUI()
  void setupGUI(){
    setLayout(new BorderLayout());//new FlowLayout());
    setBackground(Color.lightGray);

    Panel northPanel=new Panel();
    northPanel.setLayout(new FlowLayout());

    Panel centerPanel=new Panel();
    centerPanel.setLayout(new FlowLayout());

    Panel southPanel=new Panel();
    southPanel.setLayout(new FlowLayout());

    chatArea=new TextArea(null,7,56,TextArea.SCROLLBARS_VERTICAL_ONLY);
    chatArea.setBackground(Color.white);
    chatArea.setForeground(Color.black);
    chatArea.setEditable(false);

    input=new TextField(40);
    input.addActionListener(this);
    input.requestFocus();

    sendButton=new Button("Send");
    sendButton.addActionListener(this);
    sendButton.setActionCommand("send");

    closeButton=new Button("Close");
    closeButton.addActionListener(this);
    closeButton.setActionCommand("close");

    //chatArea.setText((chatArea.preferredSize()).toString());
    //northPanel.add(chatArea);

    centerPanel.add(input);
    centerPanel.add(sendButton);

    southPanel.add(closeButton);

    northPanel.add(chatArea);

    add("Center",centerPanel);
    add("South",southPanel);
    add("North",northPanel);

    pack();
    setSize(500,250);
    setResizable(true);

  }

}