/**
 *  Class Core.java
 *  @author: Anders Sandstrom
 *  UCCS 1997
 */

import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

import imgbtn.*;
import colortools.*;


public class Core implements MouseListener,
                             WindowListener,
                             ActionListener,
                             ResultProcessor
{

  public Core(){
    gui=new GUI(this);
    init();
  }

  void init(){
    connected=false;
    defaultConnectSettings=new ConnectInfo();
    getDefaultConnectSettings();
    connectInfo=defaultConnectSettings;
    dialog= new ConnectDialog2(gui,this,"Create new conference",true,connectInfo);

    infoDialog= new InfoDialog(this,gui);

    chatDialog= new ChatDialog(gui,"Chatting",false,this);


         /*System.out.println("\ndefaultConnectSettings:");
         System.out.println("userName: "+defaultConnectSettings.userName);
         System.out.println("serverIP: "+defaultConnectSettings.serverIp);
         System.out.println("serverPort: "+defaultConnectSettings.serverPort);
         System.out.println("conferenceName: "+defaultConnectSettings.conferenceName+"\n");*/

    openFile=new FileDialog(gui,"Open File",FileDialog.LOAD);
    saveFile=new FileDialog(gui,"Save File As",FileDialog.SAVE);
    file=new File(defaultConnectSettings.conferenceName+".ctf");
    //dir=new File(file.getAbsolutePath());
    saveFile.setFile(file.toString());

    //System.out.println("getName: "+file.getName());
    Scroll scroll=new Scroll("Zodega1.1",gui.leftInfoField);
    scroll.start();

  }


  /////////// load/save //////////////////////////////

  void open(){
    String hela=new String(file.getAbsolutePath());
    String halva=new String(file.getName());
    //System.out.println("hela: "+hela);
    //System.out.println("halva: "+halva);
    //System.out.println("index: "+hela.indexOf(halva));
    //System.out.println("substring: "+(file.getAbsolutePath()).substring(0,(file.getAbsolutePath()).indexOf(file.getName())));

    //System.out.println(test.substring(0,(file.getAbsolutePath()).indexOf(file.getName())));
    openFile.setDirectory((file.getAbsolutePath()).substring(0,(file.getAbsolutePath()).indexOf(file.getName())));
    openFile.setFile(file.getName());
    openFile.show();
    if(openFile.getFile()==null) return;
    file=new File(openFile.getDirectory(),openFile.getFile());
    Vector v=readFile();
    gui.canvas.clear();
    gui.canvas.setVector(v);
  }

  void save(){
    writeFile();
  }

  void saveAs(){
    saveFile.setDirectory((file.getAbsolutePath()).substring(0,(file.getAbsolutePath()).indexOf(file.getName())));
    saveFile.setFile(file.getName());
    saveFile.show();
    if(saveFile.getFile()==null) return;
    file=new File(saveFile.getDirectory(),saveFile.getFile());
    writeFile();
  }

  Vector readFile(){
    //System.out.println(file.toString());
    Vector v=new Vector();
    try{
      DataInputStream is=new DataInputStream(new FileInputStream(file));
      GraphicalObject[] in=readData(is);
      is.close();
      for(int i=0;i<in.length;i++){
        v.addElement(in[i]);
      }
      gui.canvas.setBackgroundColor(new Color(in[0].farg));
      gui.showBackgroundColor(gui.canvas.getBackgroundColor());


    }
    catch(IOException e){
      //notifyUser((String)e);
      System.out.println("Error: "+e);
      System.exit(1);
    }
    return v;
  }

  void writeFile(){
    //System.out.println(file.toString());

    try{
      PrintWriter os=new PrintWriter(new FileOutputStream(file));
      writeData(gui.canvas.getVector(),os,(gui.canvas.getBackgroundColor()).getRGB());
      os.close();
    }
    catch(IOException e){
      //notifyUser((String)e);
      System.out.println("Error: "+e);
      System.exit(1);
    }
  }

  static void writeData(Vector v, PrintWriter os,int rgb) throws IOException{
    //System.out.println(go.length);
    os.println("ctf");
    os.println(rgb);
    os.println(v.size());
    for(int i=0;i<v.size();i++){
      GraphicalObject go = (GraphicalObject) v.elementAt(i);
      //go.print();
      go.writeData(os);
    }
  }

  static GraphicalObject[] readData(DataInputStream is) throws IOException{
    String temp=is.readLine();
    //System.out.println(temp);
    if(temp.equals("ctf")){
       int rgb=(new Integer(is.readLine())).intValue();

      int n=(new Integer(is.readLine())).intValue();
      GraphicalObject[] obj=new GraphicalObject[n];
      obj[0].farg=rgb;
      for(int i=0;i<n;i++){
        obj[i]=new GraphicalObject();
        obj[i].readData(is);

      }
      return obj;
    }
    else return null;
  }

  ////////////////////////////////////////////////////////




  public void processResult(Dialog source, Object result){
    if(source instanceof ConnectDialog2) connect(result);
  }

  //Method isConnected()
  public boolean isConnected(){
    return connected;
  }

  // Method getDefaultSettings
  void getDefaultConnectSettings(){
    InetAddress localAddress=null;
    defaultConnectSettings.userName="";

    try{
      localAddress = InetAddress.getLocalHost();
    }
    catch(UnknownHostException ex){
    }

    String hostName=localAddress.getHostName();
    defaultConnectSettings.serverIp=hostName;
    defaultConnectSettings.serverPort="9005";
    defaultConnectSettings.conferenceName="Conference1";
  }

   // Method update_dosdis()
  public void update_dosdis(DataOutputStream cdos,DataInputStream  cdis,String id_nm,String conf_nm,String ip_add){
    dos = cdos;
    dis = cdis;
    id_name = id_nm;
    conf_name = conf_nm;
    ipaddress = ip_add;
    connected = true;
    gui.writeToStatusBar(RIGHT,id_name+" "+conf_name+" "+" "+ipaddress,false);
    gdis = new getdis();
    gdis.getCore(this);
    gdis.update_dis(dis,conf_nm,id_nm);

    gdis.start();
  }



  void conferenceInfo(){
    gui.writeToStatusBar(LEFT,"INFO MODE",false);
    infoDialog.setVisible(true);
  }

  void openChattingDialog(){
    chatDialog.setUserName(info.userName);
    chatDialog.setVisible(true);
  }

  void revoke(){
    String tmp_name = infoDialog.getUserName();
    String revoke_time = infoDialog.getTime();
    if(tmp_name.length() != 0 && revoke_time.length() != 0){
      revoke_client("Revoke", tmp_name, revoke_time);
    }
    infoDialog.dispose();
  }

  void suspend(){
    String tmp_name = infoDialog.getUserName();
    String revoke_time = infoDialog.getTime();
    if(tmp_name.length() != 0 && revoke_time.length() != 0){
      revoke_client("Suspend", tmp_name, revoke_time);
    }
    infoDialog.dispose();
  }

  void newConference(){
    if(!connected) gui.canvas.clear();
  }

  void about(){
    scroll=new Scroll(defaultSettings.about,gui.leftInfoField);
    scroll.start();
  }

  void exit(){
    if(!connected) System.exit(0);
  }


///////////////// connect //////////////////////////

  void fastConnect(){
    gui.writeToStatusBar(LEFT,"CONNECT MODE",false);
    //ConnectInfo connectInfo = new ConnectInfo();
    //connectInfo=defaultConnectSettings;
    //dialog= new ConnectDialog2(gui,this,"Create new conference",true,connectInfo);
    dialog.setVisible(true);
  }

  // Method connect()
  void connect(Object result){
    //System.out.println("Entering connect()");
    info=(ConnectInfo)result;

    //System.out.println("=========================================");
    //System.out.println("ConnectInfo:");
    //System.out.println("userName: "+info.userName);
    //System.out.println("serverIP: "+info.serverIp);
    //System.out.println("serverPort: "+info.serverPort);
    //System.out.println("conferenceName: "+info.conferenceName);
    //System.out.println("=========================================");

    if(!connected){
      if(connectClient(info)){
        connected=true;
        update_dosdis(dos, dis, id_name, conf_name,ipaddress);

        gui.connectImageButton.setEnabled(false);
        gui.disconnectImageButton.setEnabled(true);

        gui.chatImageButton.setEnabled(true);

        gui.fileMenu.getItem(5).setEnabled(false);
        gui.fileMenu.getItem(6).setEnabled(true);


      }
      else connected=false;
    }
  }

  // Method connectClient
  boolean connectClient(ConnectInfo info){
    //System.out.println("Entering connectClient()");
    String userName;
    String serverIp;
    String serverPort;
    String conferenceName;
    String serveripaddress = null;
    int    port  =  0;
    int temp_port;
    Socket  socket  =  null;


    if(info.userName.length() == 0 || info.serverIp.length() == 0 || info.serverPort.length() == 0 || info.conferenceName.length() == 0){
       System.out.println("ERROR: connection");
      gui.writeToStatusBar(RIGHT,"ID OR IP Address OR port OR Conference is Empty",false);
      return false;
    }
    else{
      serveripaddress = info.serverIp;
      port = new Integer(info.serverPort).intValue();
      id_name = info.userName;
      conf_name = info.conferenceName;
       //System.out.println("serveripaddress: "+serveripaddress);
       //System.out.println("port: "+port);
       //System.out.println("id_name: "+info.userName);
       //System.out.println("conf_name: "+conf_name);
    }

    try{
      socket = new Socket(serveripaddress, port);
      //System.out.println("Socket" + socket);
      dos = new DataOutputStream(socket.getOutputStream());
      dis = new DataInputStream(socket.getInputStream());
      temp_port = socket.getLocalPort();
      //System.out.println("funkar: " +serveripaddress);
    }
    catch(IOException  e){
      connected = false;
      System.out.println("Error:  IO  Exception");
      gui.writeToStatusBar(RIGHT,"Error: Cannot Connect to Server",false);
      return false;
    }

    try{
      InetAddress add = null;
      add = InetAddress.getLocalHost();

      String ipadd_str = add.toString();
      StringTokenizer inputbufferST = new StringTokenizer(ipadd_str,"/");
      String ipadd_str2 = inputbufferST.nextToken();
      ipadd_str2 = inputbufferST.nextToken();
      String ipadd = ipadd_str2+":"+Integer.toString(temp_port);
      ipaddress = ipadd;
    }
    catch(UnknownHostException e){
      connected = false;
      return false;
    }

    try{
      dos.writeBytes(conf_name + " " +
                       id_name + " " +
                       "Connect" + " " +
                       ipaddress + " " +
                       "0" + " " +
                       "\n");
      dos.flush();
      connected = true;
      gui.writeToStatusBar(RIGHT,"Connected To Server",false);
      return true;
    }
    catch(IOException  e){
      connected = false;
      System.out.println("Error:  IO  Exception");
      gui.writeToStatusBar(RIGHT,"Error: Cannot Connect to Server",false);
      return false;
    }
    //System.out.println("Leaving connectClient()");
  }


  /////////////////// disconnect ///////////////////////////

    // Method disconnectClient()
  public void disconnectClient(){
    gui.writeToStatusBar(RIGHT,"DISCONNECT MODE",false);

    send_data_to_server("close"+" "+ipaddress);
    //System.out.println("Disconnecting");
    //System.out.println("ipaddress: "+ipaddress);

    if(!connected) return;

    gui.disconnectImageButton.setEnabled(false);
    gui.connectImageButton.setEnabled(true);

    gui.chatImageButton.setEnabled(false);

    gui.fileMenu.getItem(5).setEnabled(true);
    gui.fileMenu.getItem(6).setEnabled(false);

    infoDialog.clearList();

    gui.writeToStatusBar(RIGHT," ",false);
    gdis.update_dis(null,null,null);
    gdis.stop();
    id_name = null;
    conf_name = null;
    ipaddress = null;
    connected = false;
  }


////////////////// Send //////////////////////////////////////////////////////

  /**
   *  Send a change background color message
   *
   *  @param  color Current background color.
   */
  public void sendBackgroundColorData(Color color){
	  if(!connected) return;
    int rgb=(int)color.getRGB();
    gui.writeToStatusBar(LEFT,"Sending COLOR INFO",false);
    send_data_to_server("B"+" "+rgb);
  }

  /**
   *  Send a Chatting message
   *
   *  @param  msg   Message.
   */
  public void sendChatData(String msg){
	  if(debug) System.out.println("Entering: sendChatData()");
    String name=new String(info.userName);
    if(name==null) name="local";
    //String message=new String(name+": "+msg+"\n");
    send_data_to_server("H"+" "+name+" "+msg);
    //System.out.println("Leaving: sendChatData()"+msg);
  }

  /**
   *  Send a Control Character message
   *
   *  @param  id      ID #.
   *  @param  charId  Characters index in it's string.
   *  @param  key     Control character value.
   */
  public void sendCtrlCharData(int id, int charId, int key){
	if(debug) System.out.println("Entering: sendDelCharData()");
    send_data_to_server("C"+" "+id+" "+charId+" "+key);
    if(debug) System.out.println("Leaving: sendDelCharData()");
  }

  /**
   *  Send a delete graphical object message
   *
   *  @param  id  Graphical object id.
   */
  public void sendDeleteData(int id){
    if(debug) System.out.println("Entering: sendDeleteData()");
	  if(!connected) return;
    gui.writeToStatusBar(LEFT,"Sending DELETE INFO",false);
    send_data_to_server("D"+" "+id);
    if(debug) System.out.println("Leaving: sendDeleteData()");
  }

  /**
   *  Send a graphical object message
   *
   *  @param  x1            First x coordinate.
   *  @param  y1            First y coordinate.
   *  @param  x2            Second x coordinate.
   *  @param  y2            Second y coordinate.
   *  @param  objectType    Type of graphical object.
   *  @param  objectId      ID #.
   *  @param  objectColor   Objects color.
   *  @param  fill          Solid or outlined.
   */
  public void sendPenData(int x1,int y1,int x2,int y2,int objectType,int objectId,int objectColor, int fill){
	  if(debug) System.out.println("Entering: sendPenData()");
	  if(!connected) return;
    gui.writeToStatusBar(LEFT,"Sending PEN INFO",false);
    send_data_to_server("P"+" "+x1+" "+y1+" "+x2+" "+y2+" "+objectType+" "+" "+objectId+" "+objectColor+" "+fill);
    //dos.flush();
    if(debug) System.out.println("Leaving: sendPenData()");
  }

  /**
   *  Send a changes to graphical object message
   *
   *  @param  x1            First x coordinate.
   *  @param  y1            First y coordinate.
   *  @param  x2            Second x coordinate.
   *  @param  y2            Second y coordinate.
   *  @param  objectType    Type of graphical object.
   *  @param  objectId      ID #.
   *  @param  objectColor   Objects color.
   *  @param  fill          Solid or outlined.
   */
  public void sendChangePenData(int x1,int y1,int x2,int y2,int objectType,int objectId,int objectColor, int fill){
	  if(debug) System.out.println("Entering: sendchangePenData()");
	  if(!connected) return;
    gui.writeToStatusBar(LEFT,"Sending PEN INFO",false);
    send_data_to_server("p"+" "+x1+" "+y1+" "+x2+" "+y2+" "+objectType+" "+" "+objectId+" "+objectColor+" "+fill);
    //dos.flush();
    if(debug) System.out.println("Leaving: sendchangePenData()");
  }





  /**
   *  Send a text object message
   *
   *  @param  x           X coordinate.
   *  @param  y           Y coordinate.
   *  @param  w           Character width.
   *  @param  h           Character height.
   *  @param  size        Font size.
   *  @param  style       Font style.
   *  @param  name        Font name.
   *  @param  objectType  Type of graphical object.
   *  @param  objectId    ID #.
   *  @param  charId      Characters index in it's string.
   *  @param  key         Character value.
   *  @param  objectColor Characters color.
   */
  public void sendTextData(int x,int y,int w,int h,int size,int style,String name,int objectType,int objectId,int charId,int key,int objectColor){
	  if(debug) System.out.println("Entering: sendTextData()");
	  if(!connected) return;
    gui.writeToStatusBar(LEFT,"Sending TEXT INFO",false);
    send_data_to_server("T"+" "+x+" "+y+" "+w+" "+h+" "+size+" "+style+" "+name+" "+objectType+" "+objectId+" "+charId+" "+key+" "+objectColor);
    //dos.flush();
    if(debug) System.out.println("Leaving: sendTextData()");
  }

  /**
   *  Send Changes to a text object message
   *
   *  @param  x           X coordinate.
   *  @param  y           Y coordinate.
   *  @param  w           Character width.
   *  @param  h           Character height.
   *  @param  size        Font size.
   *  @param  style       Font style.
   *  @param  name        Font name.
   *  @param  objectType  Type of graphical object.
   *  @param  objectId    ID #.
   *  @param  charId      Characters index in it's string.
   *  @param  key         Character value.
   *  @param  objectColor Characters color.
   */
  public void sendChangeTextData(int x,int y,int w,int h,int size,int style,String name,int objectType,int objectId,int charId,int key,int objectColor){
	  if(debug) System.out.println("Entering: sendTextData()");
	  if(!connected) return;
    gui.writeToStatusBar(LEFT,"Sending TEXT INFO",false);
    send_data_to_server("t"+" "+x+" "+y+" "+w+" "+h+" "+size+" "+style+" "+name+" "+objectType+" "+objectId+" "+charId+" "+key+" "+objectColor);
    //dos.flush();
    if(debug) System.out.println("Leaving: sendTextData()");
  }


  /**
   *  Revoke client
   *
   *  @param  command      Revoke or suspend.
   *  @param  revokeName   User name.
   *  @param  revokeTime   Time.
   */
  public void revoke_client(String command,String revokeName,String revokeTime){
    if((command.length() == 0) ||(revokeName.length() == 0) ||(revokeTime.length() == 0)){
      return;
    }
    send_data_to_server(command+" "+ipaddress+" "+revokeTime+" "+revokeName);
  }

  /**
   *  Send a string to server
   *
   *  @param  str   String.
   */
  public void send_data_to_server(String str){
    if(debug) System.out.println("Entering: send_data_to_server()");
    if(!connected ||(str.length() == 0) ||(id_name.length() == 0) ||(conf_name.length() == 0) ||(dos == null)){
      return;
    }
    try{
      dos.writeBytes(conf_name+" "+id_name  +" "+str+"\n");
      dos.flush();
    }
    catch(IOException e){
      System.out.println("ERROR: Writing");
    }
  }

///////////////// Recieve ////////////////////////////////////

  synchronized boolean getdis_pen(int x1,
                                  int y1,
                                  int x2,
                                  int y2,
                                  int objectType,
                                  int objectId,
                                  int objectColor,
                                  int fill)
  {
    gui.canvas.addObject(x1,y1,x2,y2,objectType,objectId,objectColor,fill);
    return true;
  }

  synchronized boolean getdis_changePen(int x1,
                                  int y1,
                                  int x2,
                                  int y2,
                                  int objectType,
                                  int objectId,
                                  int objectColor,
                                  int fill)
  {
    gui.canvas.changeObject(x1,y1,x2,y2,objectType,objectId,objectColor,fill);
    return true;
  }
  synchronized boolean getdis_text(int x,
                                   int y,
                                   int w,
                                   int h,
                                   int size,
                                   int style,
                                   String name,
                                   int type,
                                   int id,
                                   int charId,
                                   int key,
                                   int rgb)
  {
    //System.out.println("text received");
    gui.canvas.addTextObject(x,y,w,h,size,style,name,type,id,charId,key,rgb);
    return true;
  }

  synchronized boolean getdis_changeText(int x,
                                   int y,
                                   int w,
                                   int h,
                                   int size,
                                   int style,
                                   String name,
                                   int type,
                                   int id,
                                   int charId,
                                   int key,
                                   int rgb)
  {
    //System.out.println("change text received");
    gui.canvas.changeTextObject(x,y,w,h,size,style,name,type,id,charId,key,rgb);
    return true;
  }


  synchronized boolean getdis_delete(int id){
    gui.canvas.deleteObject(id);
    return true;
  }

  synchronized boolean getdis_background(int color){
    gui.canvas.setBackgroundColor(new Color(color));
    gui.showBackgroundColor(gui.canvas.getBackgroundColor());
    return true;
  }

  synchronized boolean getdis_ctrlChar(int id,int charId,int key){
    //System.out.println("deleteChar received");
    gui.canvas.deleteTextObject(id,charId,key);
    return true;
  }

  synchronized boolean getdis_chat(String name,String msg){
    gui.writeToStatusBar(LEFT,"New message from "+name+".",false);
    if(!chatDialog.isShowing()) gui.toolkit.beep();
    chatDialog.printRemote(name,msg);
    //else
    return true;
  }

  // Method getdis_list()
  synchronized void getdis_list(String[] str, int max_index){
    infoDialog.clearList();
    infoDialog.updateList(str, max_index);
    //infoList.clear();
    //infoList.update_list(str, max_index);
  }

////////////////// Events /////////////////////////////

  /**
   *  Listen to Window Closing generated in GUI.java
   *
   *  @param  event   A WindowEvent object with the
   *                  specified source window and type.
   *
   */
  public void windowClosing(WindowEvent event){
    exit();
  }

  public void windowClosed(WindowEvent event){
  }

  public void windowDeiconified(WindowEvent event){
  }

  public void windowIconified(WindowEvent event){
  }

  public void windowActivated(WindowEvent event){
  }

  public void windowDeactivated(WindowEvent event){
  }

  public void windowOpened(WindowEvent event){
  }


  /**
   *  Listen to Mouse Pressed generated in GUI.java
   *
   *  @param  event   A MouseEvent object with the
   *                  specified source component, type,
   *                  modifiers, coordinates, and click count.
   *
   */
  public void mousePressed(MouseEvent event){
    //System.out.println("event in core!");
    Object source=event.getComponent();
    if(source instanceof ImageButton){
    String name=(event.getComponent()).getName();
      if(name=="New") newConference();
      if(name=="Open") open();
      if(name=="Save"){
        //if(outFile==null) saveAs();
        if((file.getName()).equals(defaultConnectSettings.conferenceName+".ctf")) saveAs();
        else save();
      }
      if(name=="Connect") fastConnect();
      if(name=="Disconnect") disconnectClient();
      if(name=="Chatting") openChattingDialog();
    }
  }

  /**
   *  Listen to Mouse Entered generated in GUI.java
   *
   *  @param  event   A MouseEvent object with the
   *                  specified source component, type,
   *                  modifiers, coordinates, and click count.
   *
   */
  public void mouseEntered(MouseEvent event){
    Object source=event.getComponent();
    //System.out.println(source.toString());
    if(source instanceof ImageButton){
      gui.writeToStatusBar(LEFT,((ImageButton)source).getName(),false);
    }
    else if(source instanceof ColorPalette){
      gui.writeToStatusBar(LEFT,((ColorPalette)source).getName(),false);
    }
  }

  /**
   *  Listen to Mouse Exited generated in GUI.java
   *
   *  @param  event   A MouseEvent object with the
   *                  specified source component, type,
   *                  modifiers, coordinates, and click count.
   *
   */
  public void mouseExited(MouseEvent event){
    gui.undoWriteToStatusBar(LEFT);
  }

  public void mouseClicked(MouseEvent event){
  }

  public void mouseReleased(MouseEvent event){
  }



  /**
   *  Listen to Action Performed generated in GUI.java
   *  or in InfoDialog.java
   *
   *  @param  event   An ActionEvent object with the
   *                  specified source object.
   *
   */
  public void actionPerformed(ActionEvent event){
    //System.out.println("action event in core!");
    String cmd=event.getActionCommand();

    // File Menu
    if (cmd.equals("new")) newConference();

    else if(cmd.equals("open")) open();

    else if(cmd.equals("save")){
        if((file.getName()).equals(defaultConnectSettings.conferenceName+".ctf")) saveAs();
        else save();
    }
    else if(cmd.equals("saveas")) saveAs();

    else if(cmd.equals("connect")) fastConnect();

    else if(cmd.equals("disconnect")) disconnectClient();

    else if(cmd.equals("info")) conferenceInfo();

    else if(cmd.equals("exit")) exit();


    // Help menu
    else if(cmd.equals("about")) about();

    // InfoDialog

    else if(cmd.equals("revoke")) revoke();

    else if(cmd.equals("suspend")) suspend();

    else if(cmd.equals("cancel"))  infoDialog.dispose();

  }

////////////////// Declarations ///////////////////////

  private String buttonStatus;
  private boolean LEFT=true;
  private boolean RIGHT=false;
  public GUI gui;

  private ConnectInfo defaultConnectSettings;
  private ConnectInfo connectInfo;
  private ConnectInfo info;

  private boolean connected;
  private String id_name;

  private Dialog dialog;
  private ChatDialog chatDialog;
  private InfoDialog infoDialog;

  private FileDialog openFile;
  private FileDialog saveFile;

  //private InfoList infoList;

  private DataInputStream   dis  =  null;
  private DataOutputStream  dos  =  null;

  private String ipaddress  =  null;

  private getdis gdis;
  private String conf_name;
  //private int rgb;

  private DefaultSettings defaultSettings=new DefaultSettings();

  private Scroll scroll;


  //private InfoDialog infoDialog;

  //private String outFile=null;

  //private String inFile=null;

  //private File dir=null;

  private File file=null;

  //private File inFile=null;

  //private File outFile=null;


  boolean debug=false;



}

