/**
 *  Class ExtendedCanvas.java
 *  @author: Anders Sandstrom
 *  UCCS 1997
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;

import colortools.*;
import imgbtn.*;

public class ExtendedCanvas extends Canvas implements MouseListener,
                                                      ItemListener,
                                                      KeyListener,
                                                      ActionListener

{

  /**
   *  Constructor
   */
  public ExtendedCanvas(GUI gui,ResultProcessor core, int mode){
    super();
    this.gui=gui;
    this.mode=mode;
    owner=core;
    setupColors();
    setupFont();
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
    //enableEvents(AWTEvent.KEY_EVENT_MASK);
    addKeyListener(this);
    init();
  }

  /**
   *  Setup of colors.
   */
  void setupColors(){
    backgroundColor=defaultSettings.getBackgroundColor();
    foregroundColor=defaultSettings.getForegroundColor();
    setBackground(backgroundColor);
  }

  /**
   *  Setup of font.
   */
  void setupFont(){
    currentFontName=defaultSettings.getFontName(gui.getFontNames());
    currentFontSize=defaultSettings.getFontSize();
    currentFontStyle=defaultSettings.getFontStyle();
    newFont(currentFontName,currentFontStyle,currentFontSize);
  }

  /**
   *  Initialition of varialbles.
   */
  void init(){
    currentLineWidth=defaultSettings.getLineWidth();
    objects = new Vector(INITIAL_SIZE,INCREMENT);
    newObject = false;
    freehandId=0;
    firstElement=0;
    //lastElement=0;
    index=0;
    position=false;
    first=true;
    cursor=null;
    newLine=false;
    defaultObjectId= Math.abs((int)new Random().nextInt());
    currentObjectId=defaultObjectId;
  }

  /**
   *  Process local MouseEvents.
   *
   *  @param  event   A MouseEvent object with the specified source component,
   *                  type, modifiers, coordinates, and click count.
   */
  protected void processMouseEvent(MouseEvent event){
    requestFocus();
    switch(event.getID()){

      case MouseEvent.MOUSE_RELEASED:

        if(selected && draged){
            mouseX=event.getX()-mouseX;
            mouseY=event.getY()-mouseY;
            draged=false;
            BoundingBox bB1=boundingBox;//new BoundingBox(selectedId,objects,backgroundColor);

          if(((GraphicalObject)objects.elementAt(findFirstElement(selectedId))).getType()==GraphicalObject.TEXT ||
             ((GraphicalObject)objects.elementAt(findFirstElement(selectedId))).getType()==GraphicalObject.FREEHAND){
            for(int i=0;i<objects.size();i++){
              if(((GraphicalObject)objects.elementAt(i)).getId()==selectedId){
                ((GraphicalObject)objects.elementAt(i)).setStartX(mouseX+((GraphicalObject)objects.elementAt(i)).startX);
                ((GraphicalObject)objects.elementAt(i)).setStartY(mouseY+((GraphicalObject)objects.elementAt(i)).startY);
              }
            }
             }
             else{

              for(int i=0;i<objects.size();i++){
                if(((GraphicalObject)objects.elementAt(i)).getId()==selectedId){
                  ((GraphicalObject)objects.elementAt(i)).setStartX(mouseX+((GraphicalObject)objects.elementAt(i)).startX);
                  ((GraphicalObject)objects.elementAt(i)).setStartY(mouseY+((GraphicalObject)objects.elementAt(i)).startY);
                  ((GraphicalObject)objects.elementAt(i)).setEndX(mouseX+((GraphicalObject)objects.elementAt(i)).endX);
                  ((GraphicalObject)objects.elementAt(i)).setEndY(mouseY+((GraphicalObject)objects.elementAt(i)).endY);
                }
              }
            }
            boundingBox =new BoundingBox(selectedId,objects,backgroundColor);
            repaint(bB1.getX(),bB1.getY(),bB1.getW(),bB1.getH());
            repaint(boundingBox.getX(),boundingBox.getY(),boundingBox.getW(),boundingBox.getH());

            //repaint();
            sendChangeObject(selectedId,mouseX,mouseY);
        }

        else if(mode==GraphicalObject.FREEHAND){
          int lastElement=objects.size();
          //System.out.println("lastElement="+lastElement);
          for(int i=firstElement;i<lastElement;i++){
            sendObject((GraphicalObject)objects.elementAt(i));
          }
        }
        else if(newObject){
          objects.addElement(current);
          sendObject(current);
          newObject = false;
        }
      break;

      case MouseEvent.MOUSE_PRESSED:



        if(mode==GraphicalObject.SELECT){
          int id;
          if((id=findObject(event.getX(),event.getY()))>=0){
            if(selected) repaint(boundingBox.getX(),boundingBox.getY(),boundingBox.getW(),boundingBox.getH());
            selected=true;
            selectedId=id;
            boundingBox=new BoundingBox(id,objects,backgroundColor);
            repaint(boundingBox.getX(),boundingBox.getY(),boundingBox.getW(),boundingBox.getH());
            mouseX=event.getX();
            mouseY=event.getY();
          }
          else{
            selected=false;
            repaint(boundingBox.getX(),boundingBox.getY(),boundingBox.getW(),boundingBox.getH());
          }
        }
        else if(mode==GraphicalObject.TEXT){
          position=true;
          first=true;
          tempId=0;
          index=0;
          stringLength=0;
          initialX=event.getX();
          initialY=event.getY();
          newX=initialX;
          newY=initialY;
          requestFocus();
          if(cursor!=null) repaint(cursor.getX(),cursor.getY(),cursor.getW(),cursor.getH());
          cursor=new TextCursor(initialX,initialY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
          repaint(cursor.getX(),cursor.getY(),cursor.getW(),cursor.getH());
          //System.out.println("MouseDown:\nPosition: "+position+"\nx="+initialX+"\ny="+initialY);
        }
        else if(mode==GraphicalObject.FREEHAND){
          freehandId=getObjectId();
          firstElement=objects.size();
          if(debug) System.out.println("firstElement="+firstElement);
        }
        else{
          current = new GraphicalObject(mode,event.getX(),event.getY());
          current.setColor(foregroundColor);
          current.setId(getObjectId());
          current.setLineWidth(currentLineWidth);
          current.setLocal();
          if(debug) System.out.println("currentId: "+current.getId());
          newObject = true;
        }
      break;
    }
  }


  /**
   *  Process local MouseMotionEvents.
   *
   *  @param  event   A MouseEvent object with the specified source component,
   *                  type, modifiers, coordinates, and click count.
   */
  protected void processMouseMotionEvent(MouseEvent event){
    if(event.getID()==MouseEvent.MOUSE_DRAGGED){

      int x=event.getX();
      int y=event.getY();
      if(selected){
        draged=true;
      }


      else if(mode==GraphicalObject.FREEHAND){
        if((float)Math.abs(point.startX-x)>=(float)currentLineWidth*.75 || (float)Math.abs(point.startY-y)>=(float)currentLineWidth*.75){
          point = new GraphicalObject(mode,x,y);
          point.setColor(foregroundColor);
          point.setId(freehandId);
          point.setLineWidth(currentLineWidth);
          point.setLocal();
          objects.addElement(point);
          repaint(point.startX,point.startY,point.objectLineWidth,point.objectLineWidth);
        }
      }
      else if(newObject) {
        int oldX = current.endX;
        int oldY = current.endY;
        if(mode != GraphicalObject.LINE) {
          if(event.isShiftDown()){
            int difference=Math.abs(x-current.startX)-Math.abs(y-current.startY);
            if(difference>=0)y+=difference;
            else x+=Math.abs(difference);
          }
          if(x > current.startX) current.endX = x;
          if(y > current.startY) current.endY = y;
          int width = Math.max(oldX,current.endX) - current.startX + 1;
          int height = Math.max(oldY,current.endY) - current.startY + 1;
          repaint(current.startX,current.startY,width,height);
        }
        else{
          current.endX = x;
          current.endY = y;
          int startX = Math.min(Math.min(current.startX,current.endX),oldX);
          int startY = Math.min(Math.min(current.startY,current.endY),oldY);
          int endX = Math.max(Math.max(current.startX,current.endX),oldX);
          int endY = Math.max(Math.max(current.startY,current.endY),oldY);
          repaint(startX-currentLineWidth,startY-currentLineWidth,endX-startX+currentLineWidth*2,endY-startY+currentLineWidth*2);
        }
      }
    }
  }

  /**
   *  Process local KeyEvents.
   *
   *  @param  event   A KeyEvent object with the
   *                  specified source component,
   *                  type, modifiers, and key.
   */
  public void keyReleased(KeyEvent event){
    int key=event.getKeyCode();

    if(/*mode!=GraphicalObject.TEXT && */ key==KeyEvent.VK_DELETE && selected){
      if(deleteObject(selectedId)){
        //currentObjectId--;
        selected=false;
      }
      owner.sendDeleteData(selectedId);

    }

    else if(mode==GraphicalObject.TEXT && position){
      if(first){
        tempId=getObjectId();
        first=false;
      }

      if(key==KeyEvent.VK_LEFT /*Left arrow*/){
        stepLeft();
        //BoundingBox bB=new BoundingBox(tempId,objects,getColor());
        //repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
      }

      else if(key==KeyEvent.VK_RIGHT /*Right arrow*/){
        stepRight();
        //BoundingBox bB=new BoundingBox(tempId,objects,getColor());
        //repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
      }

      else if(key==KeyEvent.VK_UP /* Up arrow*/){
      }

      else if(key==KeyEvent.VK_DOWN /*Down arrow*/){
      }

      else if(key==KeyEvent.VK_ENTER /*Enter*/ ){
        enter();
        BoundingBox bB=new BoundingBox(tempId,objects,getColor());
        repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
        //repaint();
        newLine=true;
      }

      else if(key==KeyEvent.VK_BACK_SPACE /*Backspace*/){
        removeChar(key);
        //BoundingBox bB=new BoundingBox(tempId,objects,getColor());
        //repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
        repaint();
      }

      else if(key==KeyEvent.VK_DELETE/*127Delete*/){
        deleteChar(key);
        BoundingBox bB=new BoundingBox(tempId,objects,getColor());
        repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
      }

      else if(Character.isLetterOrDigit((char)key) || key==KeyEvent.VK_SPACE){
        if(event.getModifiers()==KeyEvent.SHIFT_MASK)
          addChar(key,newLine);
        else addChar((int)Character.toLowerCase((char)key),newLine);
        BoundingBox bB=new BoundingBox(tempId,objects,getColor());
        repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
      }
    }
  }

  public void keyTyped(KeyEvent event){

  }
  public void keyPressed(KeyEvent event){

  }



  int findObject(int xPos, int yPos){
    int id=-1;
    for(int i=0;i<objects.size();i++){
      GraphicalObject obj=(GraphicalObject)objects.elementAt(i);
      if(obj.getType()==GraphicalObject.TEXT){
        int x1=obj.startX;
        int y1=obj.startY;
        int x2=obj.startX;
        int y2=obj.startY;
        if(xPos>x1 && yPos<y1 && xPos<x1+x2 && yPos>y1-y2) id=obj.getId();
      }
      else{
        int x1=Math.min(obj.startX,obj.endX);
        int y1=Math.min(obj.startY,obj.endY);
        int x2=Math.max(obj.startX,obj.endX);
        int y2=Math.max(obj.startY,obj.endY);
        if(obj.getType()==GraphicalObject.FREEHAND){
            if(xPos>=x1 && yPos>=y1 && xPos<=(x1+x2) && yPos<=(y1+y2)) id=obj.getId();
        }
        else{
          if(xPos>=x1 && yPos>=y1 && (x2)>=xPos && (y2)>=yPos) id=obj.getId();
        }
        //System.out.println("findObject:\n"+"x1="+x1+"\ty1="+y1+"\tx2="+x2+"\ty2="+y2);
      }
    }
    //System.out.println("Object found: "+id);
    return id;
  }




  /**
   *  Move cursor one step to the left.
   */
  void stepLeft(){
    if(index==0) return;
    index--;
    int pos=findElement(tempId,index);
    cursor=new TextCursor(((GraphicalObject)objects.elementAt(pos)).startX,((GraphicalObject)objects.elementAt(pos)).startY,((GraphicalObject)objects.elementAt(pos)).endX,((GraphicalObject)objects.elementAt(pos)).endY,backgroundColor);
    //System.out.println("<-");
    if(index>=1){
      BoundingBox bB=new BoundingBox(tempId,objects,getColor());
      repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
    }else repaint();

    //if(debug)System.out.println("pos: "+pos+" index: "+index+" stringLength: "+stringLength+" tempId: "+tempId);
  }

  /**
   *  Move cursor one step to the right.
   */
  void stepRight(){
    if(stringLength==0) return;
    if(index<stringLength) index++;
    int pos=findElement(tempId,index);
    if(pos>=0) cursor=new TextCursor(((GraphicalObject)objects.elementAt(pos)).startX,((GraphicalObject)objects.elementAt(pos)).startY,((GraphicalObject)objects.elementAt(pos)).endX,((GraphicalObject)objects.elementAt(pos)).endY,backgroundColor);
    else{
      pos=findLastElement(tempId);
      cursor=new TextCursor(((GraphicalObject)objects.elementAt(pos)).startX+fontMetrics.charWidth('a'),((GraphicalObject)objects.elementAt(pos)).startY,((GraphicalObject)objects.elementAt(pos)).endX,((GraphicalObject)objects.elementAt(pos)).endY,backgroundColor);
    }
    //System.out.println("->");
    //System.out.println("pos: "+pos+" index: "+index+" stringLength: "+stringLength+" tempId: "+tempId);
    BoundingBox bB=new BoundingBox(tempId,objects,getColor());
    repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());

  }

  /**
   *  Handel the function of pressed enter key, in text mode.
   */
  void enter(){
    int pos=0,height=0,distance=0;
    height=fontMetrics.getHeight();
    if(index==0){
      newX=initialX;
      newY=initialY;
    }
    else if(stringLength==0){
      newY+=height;
      //newX=initialX;
      cursor=new TextCursor(newX,newY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
    }
    else if(index==stringLength){
      newX=initialX;
      pos=findLastElement(tempId);
      newY=((GraphicalObject)objects.elementAt(pos)).startY+height;
      cursor=new TextCursor(newX,newY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
    }
    else{
      newX=initialX;
      pos=findElement(tempId,index);
      distance=((GraphicalObject)objects.elementAt(pos)).startX-initialX;
      for(int i=pos;i<objects.size();i++){
        if(tempId==((GraphicalObject)objects.elementAt(i)).getId()){
          ((GraphicalObject)objects.elementAt(i)).setStartY(((GraphicalObject)objects.elementAt(i)).startY+height);
          if(((GraphicalObject)objects.elementAt(i)).startY==((GraphicalObject)objects.elementAt(pos)).startY)
            ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(i)).startX-distance);
        }
      }
      cursor=new TextCursor(((GraphicalObject)objects.elementAt(pos)).startX,((GraphicalObject)objects.elementAt(pos)).startY,((GraphicalObject)objects.elementAt(pos)).endX,((GraphicalObject)objects.elementAt(pos)).endY,backgroundColor);
      sendCtrlChar(tempId,index,KeyEvent.VK_ENTER/*enter*/);

    }
  }

  /**
   *  Handel the function of pressed delete key, in text mode.
   *
   *  @param key     key pressed
   */
  void deleteChar(int key){
    int pos=0,width=0,cursorPos=0;
    if(stringLength==0 || stringLength==index)return;// Empty String or behind last letter
    pos=findElement(tempId,index);
    if(index==0 && stringLength==1)// first letter
      cursor =new TextCursor(initialX,initialY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
    else if(pos==findLastElement(tempId)){// last letter
      cursorPos=pos;
      cursor=new TextCursor(((GraphicalObject)objects.elementAt(cursorPos)).startX,((GraphicalObject)objects.elementAt(cursorPos)).startY,((GraphicalObject)objects.elementAt(cursorPos)).endX,((GraphicalObject)objects.elementAt(cursorPos)).endY,backgroundColor);
    }
    else{
      cursorPos=findElement(tempId,index+1);
      if(cursorPos==-1) cursorPos=pos;
      width=((GraphicalObject)objects.elementAt(pos)).endX;
      for(int i=pos;i<objects.size();i++){
        if(tempId==((GraphicalObject)objects.elementAt(i)).getId()){
          ((GraphicalObject)objects.elementAt(i)).setCharId(((GraphicalObject)objects.elementAt(i)).getCharId()-1);
          if(((GraphicalObject)objects.elementAt(i)).startY==((GraphicalObject)objects.elementAt(pos)).startY)
            ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(i)).startX-width);
        }
      }
      cursor=new TextCursor(((GraphicalObject)objects.elementAt(cursorPos)).startX,((GraphicalObject)objects.elementAt(cursorPos)).startY,((GraphicalObject)objects.elementAt(cursorPos)).endX,((GraphicalObject)objects.elementAt(cursorPos)).endY,backgroundColor);
    }
    objects.removeElementAt(pos);
    sendCtrlChar(tempId,index,key);
    ////sendDeleteChar(tempId,index);
    stringLength--;
    if(debug)System.out.println("pos: "+pos+" index: "+index+" stringLength: "+stringLength+" tempId: "+tempId);
  }

  /**
   *  Handels the function of pressed backspace key, in text mode.
   *
   *  @param key     key pressed
   */
  void removeChar(int key){
    int pos=0,width=0,height=0,cursorPos=0;
    height=fontMetrics.getHeight();
    pos=findElement(tempId,index);
    if(index==0) return;

    else if(cursor.baseLineX==initialX){
      if(index==stringLength){
        cursorPos=findLastElement(tempId);
        cursor=new TextCursor(((GraphicalObject)objects.elementAt(cursorPos)).startX+((GraphicalObject)objects.elementAt(cursorPos)).endX,((GraphicalObject)objects.elementAt(cursorPos)).startY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
        return;
      }
      else{
        int tempY=((GraphicalObject)objects.elementAt(pos)).startY;
        int pos1=findElement(tempId,index-1);
        for(int i=pos;i<objects.size();i++){
          if(((GraphicalObject)objects.elementAt(i)).startY==tempY){
            ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(pos1)).startX+((GraphicalObject)objects.elementAt(pos1)).endX-initialX+((GraphicalObject)objects.elementAt(i)).startX);
            ((GraphicalObject)objects.elementAt(i)).setStartY(((GraphicalObject)objects.elementAt(pos1)).startY);
          }
          else{
            ((GraphicalObject)objects.elementAt(i)).setStartY(((GraphicalObject)objects.elementAt(i)).startY-height);
          }
        }
        cursorPos=pos;
        cursor=new TextCursor(((GraphicalObject)objects.elementAt(cursorPos)).startX,((GraphicalObject)objects.elementAt(cursorPos)).startY,((GraphicalObject)objects.elementAt(cursorPos)).endX,((GraphicalObject)objects.elementAt(cursorPos)).endY,backgroundColor);
        sendCtrlChar(tempId,index,99 /*Remove line*/);

        return;
      }
    }
    else if(index==stringLength){
      pos=findLastElement(tempId);
      cursorPos=pos;
      cursor=new TextCursor(((GraphicalObject)objects.elementAt(cursorPos)).startX,((GraphicalObject)objects.elementAt(cursorPos)).startY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
      objects.removeElementAt(pos);
      sendCtrlChar(tempId,index,key /*Backspace*/);
      index--;
      stringLength--;
      return;
    }
    else{
      //System.out.println("DataData");
      pos=findElement(tempId,index-1);
      cursorPos=pos;
      width=((GraphicalObject)objects.elementAt(pos)).endX;
      for(int i=pos;i<objects.size();i++){
        if(tempId==((GraphicalObject)objects.elementAt(i)).getId()){
          ((GraphicalObject)objects.elementAt(i)).setCharId(((GraphicalObject)objects.elementAt(i)).getCharId()-1);
          if(((GraphicalObject)objects.elementAt(i)).startY==((GraphicalObject)objects.elementAt(pos)).startY)
            ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(i)).startX-width);
        }
      }
      objects.removeElementAt(pos);
      sendCtrlChar(tempId,index,key /*Backspace*/);
      index--;
      stringLength--;
      cursor=new TextCursor(((GraphicalObject)objects.elementAt(cursorPos)).startX,((GraphicalObject)objects.elementAt(cursorPos)).startY,((GraphicalObject)objects.elementAt(cursorPos)).endX,((GraphicalObject)objects.elementAt(cursorPos)).endY,backgroundColor);
      return;
    }
  }

  /**
   *  Add a new character or digit to the screen, in text mode.
   *
   *  @param  key     key pressed
   *  @param  enter   signals that the enter key has ben pressed.
   */
  void addChar(int key,boolean enter){
    int pos=0,width=0;
    width=fontMetrics.charWidth(key);
    if(enter){
      newX=initialX;
      newLine=false;
    }
    else if(index==0 && stringLength==0){
      newX=initialX;
      newY=initialY;
    }
    else if(index==stringLength){
      pos=findLastElement(tempId);
      newX=((GraphicalObject)objects.elementAt(pos)).startX+((GraphicalObject)objects.elementAt(pos)).endX;
      newY=((GraphicalObject)objects.elementAt(pos)).startY;
    }
    else{
      pos=findElement(tempId,index);
      newX=((GraphicalObject)objects.elementAt(pos)).startX;
      newY=((GraphicalObject)objects.elementAt(pos)).startY;
    }
    textObject = new GraphicalObject(mode,newX,newY,width,fontMetrics.getAscent());
    textObject.setColor(foregroundColor);
    textObject.setId(tempId);
    textObject.setCharId(index);
    textObject.setChar(key);
    textObject.setFont(currentFont);
    textObject.setLocal();

    if(index==stringLength/*last*/){
      objects.addElement(textObject);
      sendObject(textObject);
      cursor=new TextCursor(newX+width,newY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
    }
    else{
      for(int i=pos;i<objects.size();i++){
        if(tempId==((GraphicalObject)objects.elementAt(i)).getId()){
          ((GraphicalObject)objects.elementAt(i)).setCharId(((GraphicalObject)objects.elementAt(i)).getCharId()+1);
          if(((GraphicalObject)objects.elementAt(i)).startY==((GraphicalObject)objects.elementAt(pos)).startY)
            ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(i)).startX+width);
        }
      }
      objects.insertElementAt(textObject,pos);
      sendObject(textObject);
      pos=findElement(tempId,index+1);
      cursor=new TextCursor(((GraphicalObject)objects.elementAt(pos)).startX,((GraphicalObject)objects.elementAt(pos)).startY,((GraphicalObject)objects.elementAt(pos)).endX,((GraphicalObject)objects.elementAt(pos)).endY,backgroundColor);
    }
    index++;
    stringLength++;



    if(debug){
        /*for(int i=0;i<objects.size();i++){
          System.out.println("\nobjects.elementAt["+i+"]:");
          System.out.println("\tID: "+((GraphicalObject)objects.elementAt(i)).getId());
          System.out.println("\tCharID: "+((GraphicalObject)objects.elementAt(i)).getCharId());
          System.out.println("\tChar: "+((GraphicalObject)objects.elementAt(i)).getChar());
          System.out.println("\tstartX: "+((GraphicalObject)objects.elementAt(i)).startX);
          System.out.println("\tstartY: "+((GraphicalObject)objects.elementAt(i)).startY);
          System.out.println("\tendX: "+((GraphicalObject)objects.elementAt(i)).endX);
          System.out.println("\tendY: "+((GraphicalObject)objects.elementAt(i)).endY);
        }*/
    }
    if(debug)System.out.println("pos: "+pos+" index: "+index+" stringLength: "+stringLength+" tempId: "+tempId);
  }

  /**
   * Delete  id GraphicalObject on local canvas.
   *
   * @param   id      The objects unique identifier
   *
   */
  public boolean deleteObject(int id){
    boolean deleted=false;
    boolean local=false;
    if(!isElement(id))return false;
    BoundingBox bB=new BoundingBox(id,objects,getColor());
    if(debug) System.out.println("id of object to remove: "+id);
    if(objects.isEmpty()){
      if(debug) System.out.println("objects is empty");
      return false;
    }
    else{
      for(int i=objects.size()-1;i>=0;i--){
        if(((GraphicalObject)objects.elementAt(i)).getId()==id){
          if(((GraphicalObject)objects.elementAt(i)).getType()==GraphicalObject.TEXT) cursor=null;
          local=((GraphicalObject)objects.elementAt(i)).isLocal();
          objects.removeElementAt(i);
          deleted=true;
        }
      }
      repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
      //repaint(boundingBox.getX(),boundingBox.getY(),boundingBox.getW(),boundingBox.getH());
      //else repaint();//repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
      //System.out.println("objects.size(): "+objects.size());
      return deleted;
    }
  }


  boolean isElement(int id){
    for(int i=0;i<objects.size();i++){
      if(id==((GraphicalObject)objects.elementAt(i)).getId()){
        return true;
      }
    }
    return false;
  }


  /**
   *  Find the vector element described by Id and charId.
   *
   *  @param    id       Id of the GraphicalObject
   *  @param    charId   Id of the character.
   *  @return   pos      Return the vector position of the element
   *                     described by id and charId
   */
  int findElement(int id, int charId){
    int pos=-1;
    for(int i=0;i<objects.size();i++){
      if(id==((GraphicalObject)objects.elementAt(i)).getId()){
        if(charId==((GraphicalObject)objects.elementAt(i)).getCharId()) pos=i;
      }
    }
    if(debug)System.out.println("pos: "+pos);
    return pos;
  }

  /**
   * Find the first vector element whith the specific Id.
   *
   * @param   id      Id of the GraphicalObject
   * @return  pos     Return the vector position of the element
   *                  described by id.
   */
  int findFirstElement(int id){
    for(int i=0;i<objects.size();i++){
      if(id==((GraphicalObject)objects.elementAt(i)).getId()) return i;
    }
    return 0;
  }

  /**
   *  Find the the last vector element whith the specific Id.
   *
   *  @param    id       Id of the GraphicalObject
   *  @return   pos      Return the vector position of the element
   *                    described by id.
   */
  int findLastElement(int id){
    int pos=0;
    for(int i=0;i<objects.size();i++){
      if(id==((GraphicalObject)objects.elementAt(i)).getId()) pos=i;
    }
    if(debug)System.out.println("pos: "+pos);
    return pos;
  }

  /**
   * Find the # of elements with the specified id.
   *
   * @param   id      Id of the GraphicalObject
   * @return  length  Returns the # of elements with the
   *                  specified id
   */
  int stringSize(int id){
    int length=0;
    for(int i=0;i<objects.size();i++){
      if(id==((GraphicalObject)objects.elementAt(i)).getId()) length+=1;
    }
    return length;
  }



////////////////////////////////////////////////////////////////////////////////////////////////////


  /**
   * Add a remote GraphicalObject to local canvas.
   *
   * @param   x1      The objects upper right corner
   * @param   y1      The objects upper right corner
   * @param   x2      Second x coordinat, used as lower left corner or width
   * @param   y2      Second y coordinat, used as lower left corner or width
   * @param   tool    Specifies object type
   * @param   id      The objects unique identifier
   * @param   RGB     The objects color
   * @param   width   The objects line width
   *
   */
  public void addObject(int x1, int y1, int x2, int y2, int tool,int id, int RGB, int width){
    remoteObject = new GraphicalObject(tool,id,x1,y1,x2,y2,width);
    Color color=new Color(RGB);
    remoteObject.setColor(color);
    remoteObject.setId(id);
    remoteObject.setLineWidth(width);
    if(debug) System.out.println("remoteObjectId: "+remoteObject.getId());
    objects.addElement(remoteObject);
    repaint(remoteObject.startX,remoteObject.startY,remoteObject.endX,remoteObject.endY);
  }

  /**
   * Change a remote GraphicalObject to local canvas.
   *
   * @param   x1      The objects upper right corner
   * @param   y1      The objects upper right corner
   * @param   x2      Second x coordinat, used as lower left corner or width
   * @param   y2      Second y coordinat, used as lower left corner or width
   * @param   tool    Specifies object type
   * @param   id      The objects unique identifier
   * @param   RGB     The objects color
   * @param   width   The objects line width
   *
   */
  public void changeObject(int x1, int y1, int x2, int y2, int tool,int id, int RGB, int width){
    Color color=new Color(RGB);
    //remoteObject = new GraphicalObject(tool,id,x1,y1,x2,y2,width);
    BoundingBox bB1=new BoundingBox(id,objects,getColor());
    for(int i=0;i<objects.size();i++){
      if(((GraphicalObject)objects.elementAt(i)).getId()==id){
        ((GraphicalObject)objects.elementAt(i)).setColor(color);
        ((GraphicalObject)objects.elementAt(i)).setLineWidth(width);
        ((GraphicalObject)objects.elementAt(i)).setStartX(x1+((GraphicalObject)objects.elementAt(i)).startX);
        ((GraphicalObject)objects.elementAt(i)).setStartY(y1+((GraphicalObject)objects.elementAt(i)).startY);
        ((GraphicalObject)objects.elementAt(i)).setEndX(x2+((GraphicalObject)objects.elementAt(i)).endX);
        ((GraphicalObject)objects.elementAt(i)).setEndY(y2+((GraphicalObject)objects.elementAt(i)).endY);
      }
    }
    //if(debug) System.out.println("remoteObjectId: "+remoteObject.getId());
    BoundingBox bB2=new BoundingBox(id,objects,getColor());
    repaint(bB1.getX(),bB1.getY(),bB1.getW(),bB1.getH());
    repaint(bB2.getX(),bB2.getY(),bB2.getW(),bB2.getH());

  }

  /**
   * Add a remote "Text" GraphicalObject to local canvas.
   *
   * @param   x       The x coordinate for the base line of the character
   * @param   y       The y coordinate for the base line of the character
   * @param   w       The width of the character
   * @param   h       The height of the character
   * @param   size    The font size
   * @param   style   The font style
   * @param   name    The font name
   * @param   tool    Specifies object type
   * @param   id      The objects unique identifier
   * @param   charId  The characters unique identifier
   * @param   key     The value of the character
   * @param   RGB     The objects color
   *
   */
  public void addTextObject(int x, int y, int w, int h,int size,int style,String name, int type,int id,int charId,int key, int rgb ){
      int pos;
      if(debug)System.out.println("canvas.addTextObject");
      remoteTextObject = new GraphicalObject(type,x,y,w,h);
      Color color=new Color(rgb);
      Font tempFont =new Font(name,style,size);
      remoteTextObject.setColor(color);
      remoteTextObject.setId(id);
      remoteTextObject.setCharId(charId);
      remoteTextObject.setChar(key);
      remoteTextObject.setFont(tempFont);

      pos=findElement(id,charId);

      if(charId==stringSize(id)) objects.addElement(remoteTextObject);
      else{
        for(int i=pos;i<objects.size();i++){
          if(id==((GraphicalObject)objects.elementAt(i)).getId()){
            ((GraphicalObject)objects.elementAt(i)).setCharId(((GraphicalObject)objects.elementAt(i)).getCharId()+1);
            if(((GraphicalObject)objects.elementAt(i)).startY==y) ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(i)).startX+w);
          }
        }
        objects.insertElementAt(remoteTextObject,pos);
      }
      if(debug) System.out.println("remoteTextObjectId: "+remoteTextObject.getId());
      repaint();//remoteTextObject.startX,remoteTextObject.startY,remoteTextObject.endX,remoteTextObject.endY);
  }

  /**
   * change a remote "Text" GraphicalObject to local canvas.
   *
   * @param   x       The x coordinate for the base line of the character
   * @param   y       The y coordinate for the base line of the character
   * @param   w       The width of the character
   * @param   h       The height of the character
   * @param   size    The font size
   * @param   style   The font style
   * @param   name    The font name
   * @param   tool    Specifies object type
   * @param   id      The objects unique identifier
   * @param   charId  The characters unique identifier
   * @param   key     The value of the character
   * @param   RGB     The objects color
   *
   */
  public void changeTextObject(int x, int y, int w, int h,int size,int style,String name, int type,int id,int charId,int key, int rgb ){
    if(debug)System.out.println("canvas.changeTextObject");
    Color color=new Color(rgb);
    BoundingBox bB1=new BoundingBox(id,objects,getColor());
    for(int i=0;i<objects.size();i++){
      if(((GraphicalObject)objects.elementAt(i)).getId()==id){
        ((GraphicalObject)objects.elementAt(i)).setColor(color);
        ((GraphicalObject)objects.elementAt(i)).setStartX(x+((GraphicalObject)objects.elementAt(i)).startX);
        ((GraphicalObject)objects.elementAt(i)).setStartY(y+((GraphicalObject)objects.elementAt(i)).startY);
        //((GraphicalObject)objects.elementAt(i)).setEndX(w);
        //((GraphicalObject)objects.elementAt(i)).setEndY(h);
      }
    }
    //if(debug) System.out.println("remoteTextObjectId: "+remoteTextObject.getId());
    BoundingBox bB2=new BoundingBox(id,objects,getColor());
    repaint(bB1.getX(),bB1.getY(),bB1.getW(),bB1.getH());
    repaint(bB2.getX(),bB2.getY(),bB2.getW(),bB2.getH());

    //repaint();//remoteTextObject.startX,remoteTextObject.startY,remoteTextObject.endX,remoteTextObject.endY);
  }


  /**
   * Handle remote backspace, delete and enter action on local canvas.
   *
   * @param   id      The objects unique identifier
   * @param   charId  The characters unique identifier
   * @param   key     The value of backspace, delete or enter
   *
   */
  public void deleteTextObject(int id, int charId, int key){
    int pos=0,width,height;
    //if(!isElement(id)) return;
    height=fontMetrics.getHeight();
    if(debug) System.out.println("canvas.deleteTextObject");
    if(debug) System.out.println("remote id: "+id);
    if(debug) System.out.println("remote charId: "+charId);
    if(debug) System.out.println("remote key: "+key);
    if(key==KeyEvent.VK_DELETE/*Delete*/){
      pos=findElement(id,charId);
      if(debug) System.out.println("pos: "+pos);
      width=((GraphicalObject)objects.elementAt(pos)).endX;
      for(int i=pos;i<objects.size();i++){
        if(id==((GraphicalObject)objects.elementAt(i)).getId()){
          ((GraphicalObject)objects.elementAt(i)).setCharId(((GraphicalObject)objects.elementAt(i)).getCharId()-1);
          if(((GraphicalObject)objects.elementAt(i)).startY==((GraphicalObject)objects.elementAt(pos)).startY)
            ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(i)).startX-width);
        }
      }
      BoundingBox bB=new BoundingBox(id,objects,getColor());
      objects.removeElementAt(pos);
      //repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
      repaint();
    }
    else if(key==KeyEvent.VK_BACK_SPACE/*Backspace*/){
      if(charId==stringSize(id)){
        pos=findLastElement(id);
        if(debug) System.out.println("pos: "+pos);
      }
      else{
        pos=findElement(id,charId-1);
        if(debug) System.out.println("pos: "+pos);
        width=((GraphicalObject)objects.elementAt(pos)).endX;
        for(int i=pos;i<objects.size();i++){
          if(id==((GraphicalObject)objects.elementAt(i)).getId()){
            ((GraphicalObject)objects.elementAt(i)).setCharId(((GraphicalObject)objects.elementAt(i)).getCharId()-1);
            if(((GraphicalObject)objects.elementAt(i)).startY==((GraphicalObject)objects.elementAt(pos)).startY)
              ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(i)).startX-width);
          }
        }
      }
      objects.removeElementAt(pos);
      repaint();
    }
    else if(key==99){
      pos=findElement(id,charId);
      int tempY=((GraphicalObject)objects.elementAt(pos)).startY;
      int pos1=findElement(id,charId-1);
      int firstX=((GraphicalObject)objects.elementAt(findFirstElement(id))).startX;
      for(int i=pos;i<objects.size();i++){
        if(((GraphicalObject)objects.elementAt(i)).startY==tempY){
          ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(pos1)).startX+((GraphicalObject)objects.elementAt(pos1)).endX-firstX+((GraphicalObject)objects.elementAt(i)).startX);
          ((GraphicalObject)objects.elementAt(i)).setStartY(((GraphicalObject)objects.elementAt(pos1)).startY);
        }
        else ((GraphicalObject)objects.elementAt(i)).setStartY(((GraphicalObject)objects.elementAt(i)).startY-height);
      }
      BoundingBox bB=new BoundingBox(id,objects,getColor());
      //repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
      repaint();
    }
    else if(key==KeyEvent.VK_ENTER/*Enter*/){
      pos=findElement(id,charId);
      int newX=((GraphicalObject)objects.elementAt(findFirstElement(id))).startX;
      int distance=((GraphicalObject)objects.elementAt(pos)).startX-newX;
      for(int i=pos;i<objects.size();i++){
        if(id==((GraphicalObject)objects.elementAt(i)).getId()){
          ((GraphicalObject)objects.elementAt(i)).setStartY(((GraphicalObject)objects.elementAt(i)).startY+height);
          if(((GraphicalObject)objects.elementAt(i)).startY==((GraphicalObject)objects.elementAt(pos)).startY)
            ((GraphicalObject)objects.elementAt(i)).setStartX(((GraphicalObject)objects.elementAt(i)).startX-distance);
        }
      }
      BoundingBox bB=new BoundingBox(id,objects,getColor());
      //repaint(bB.getX(),bB.getY(),bB.getW(),bB.getH());
      repaint();
    }
  }

/////////////////////////////////////////////////////////////////////////////////////////////////////////


  /**
   *  Set new vector of GraphicalObjects
   *
   *  @param  v    Vector of GraphicalObjects.
   */
  public void setVector(Vector v){
    clear();
    objects=v;
    repaint();
  }

  /**
   *  Get vector containing GraphicalObjects
   *
   *  @return  objects   Vector of GraphicalObjects.
   */
  public Vector getVector(){
    return objects;
  }

  /**
   *  Set foreground color
   *
   *  @param  color   Foreground color.
   */
  public void setColor(Color color){
    foregroundColor=color;
  }

  /**
   *  Get foreground color
   *
   *  @return  foregroundColor   Current Foreground color.
   */
  public Color getColor(){
    return foregroundColor;
  }

  /**
   *  Set background color
   *
   *  @param  color   Background color.
   */
  public void setBackgroundColor(Color color){
    backgroundColor=color;
    setBackground(backgroundColor);
    repaint();
  }

  /**
   *  Get background color
   *
   *  @return backgroundColor   Current BackgroundColor.
   */
  public Color getBackgroundColor(){
    return backgroundColor;
  }

  /**
   *  Change font
   *
   *  @param  name    Font name.
   */
  public void changeFontName(String name){
    newFont(name,currentFontStyle,currentFontSize);
  }

  /**
   *  Change font style
   *
   *  @param  style    Font style (Plain, Bold, Italic).
   */
  public void changeFontStyle(int style){
    newFont(currentFontName,style,currentFontSize);
  }

  /**
   *  Change font size
   *
   *  @param  size    Font size.
   */
  public void changeFontSize(int size){
    newFont(currentFontName,currentFontStyle,size);
  }

  /**
   *  Create new font
   *
   *  @param  name    Font name.
   *  @param  style   Font style.
   *  @param  size    Font size.
   */
  public void newFont(String name, int style, int size){
    currentFont=new Font(name,style,size);
    fontMetrics=getFontMetrics(currentFont);
  }

  /**
   *  A new Id for the next GraphicalObject
   *
   *  @return  currentObjectId    A new uniq Id.
   */
  int getObjectId(){
    return currentObjectId++;
  }

  /**
   *  Toggel Italic font style on and of
   *
   */
  void setItalic(){
    if(currentFont.isItalic()) currentFontStyle-=Font.ITALIC;
    else currentFontStyle+=Font.ITALIC;
    newFont(currentFontName,currentFontStyle,currentFontSize);
    //System.out.println(currentFont.toString());
  }

  /**
   *  Toggel Bold font style on and of
   *
   */
  void setBold(){
    if(currentFont.isBold()) currentFontStyle-=Font.BOLD;
    else currentFontStyle+=Font.BOLD;
    newFont(currentFontName,currentFontStyle,currentFontSize);
    //System.out.println(currentFont.toString());
  }

  /**
   *  Deliver a GraphicalObject
   *
   *  @param  obj   An instance of GraphicalObject.
   */
  void sendObject(GraphicalObject obj){
    if(owner.isConnected()){
      if(mode==GraphicalObject.TEXT){
        int size=((Font)(obj.objectFont)).getSize();
        int style=((Font)(obj.objectFont)).getStyle();
        String name=((Font)(obj.objectFont)).getName();
        owner.sendTextData(obj.startX,obj.startY,obj.endX,obj.endY,size,style,name,obj.type,obj.getId(),obj.getCharId(),(int)obj.character,(int)(obj.getColor()).getRGB());
      }
      else owner.sendPenData(obj.startX,obj.startY,obj.endX,obj.endY,obj.type,obj.getId(),(int)(obj.getColor()).getRGB(),obj.getLineWidth());
    }
  }

  /**
   *  Deliver changes to GraphicalObject
   *
   *  @param  obj   An instance of GraphicalObject.
   */
  void sendChangeObject(int id, int deltaX, int deltaY){
    GraphicalObject obj=(GraphicalObject)objects.elementAt(findFirstElement(id));
    if(owner.isConnected()){
      if(obj.getType()==GraphicalObject.TEXT){
        int size=((Font)(obj.objectFont)).getSize();
        int style=((Font)(obj.objectFont)).getStyle();
        String name=((Font)(obj.objectFont)).getName();
        owner.sendChangeTextData(deltaX,deltaY,0,0,size,style,name,obj.type,obj.getId(),obj.getCharId(),(int)obj.character,(int)(obj.getColor()).getRGB());
      }
      else if(obj.getType()==GraphicalObject.FREEHAND){
        owner.sendChangePenData(deltaX,deltaY,0,0,obj.type,obj.getId(),(int)(obj.getColor()).getRGB(),obj.getLineWidth());
      }
      else owner.sendChangePenData(deltaX,deltaY,deltaX,deltaY,obj.type,obj.getId(),(int)(obj.getColor()).getRGB(),obj.getLineWidth());
    }
  }


  /**
   *  Deliver a control character to the Class set as owner
   *
   *  @param  id      The id of the group of GraphicalObjects
   *                  to manipulate.
   *  @param  charId  The id of the the Graphical Object within
   *                  the group.
   *  @param  key     The Character.
   */
  void sendCtrlChar(int id, int charId, int key){
    if(debug)System.out.println("sendDeleteChar");
    if(debug)System.out.println("id"+id);
    if(debug)System.out.println("charId"+charId);
    owner.sendCtrlCharData(id, charId,key);
    ////owner.sendDelCharData(id, charId);
  }

  void clear() {
    objects.removeAllElements();
    setBackgroundColor(defaultSettings.getBackgroundColor());
    gui.showBackgroundColor(getBackgroundColor());
    cursor=null;
    selected=false;
    repaint();
  }

  void print(){
    PrintJob printJob=getToolkit().getPrintJob(gui,"Printing Canvas",null);
    if(printJob!=null){
      Graphics pg=printJob.getGraphics();
      if(pg!=null){
        printAll(pg);
        pg.dispose();
      }
      printJob.end();
    }
  }

  /**
   * Override awt.Component.paint() to paint the current GraphicalObject.
   *
   * @param     g     The Graphics in which to draw
   */
  public void paint(Graphics g) {
    int numObjects = objects.size();
    for(int i=0;i<numObjects;++i) {
      GraphicalObject obj = (GraphicalObject) objects.elementAt(i);
      obj.draw(g);
    }
    if(newObject) current.draw(g);
    if(cursor!=null)cursor.draw(g);

    if(selected){
      boundingBox.draw(g);
    }
  }

  /**
   *  Listen to Item Events generated in GUI.java
   *
   *  @param  event   An instance of item event
   *                  emitted by ItemSelectable objects.
   *                  This event is generated when an
   *                  item is selected or de-selected
   *
   */
  public void itemStateChanged(ItemEvent event){
    String name;
    Object source=event.getSource();
    //System.out.println(source.toString());
    if(source instanceof Choice){
      name=((Choice)source).getName();
    }
    else {
      name=((CheckboxMenuItem)source).getName();
    }

    // Font Menu
    if(name=="fontName"){
      if(event.getStateChange()==ItemEvent.SELECTED){
        currentFontName=(String)((CheckboxMenuItem)source).getLabel();
        newFont(currentFontName,currentFontStyle,currentFontSize);
        //System.out.println(currentFont.toString());
      }
    }
    // Style Menu
    else if(name=="fontStyle"){
      if(event.getStateChange()==ItemEvent.SELECTED){
        //currentFontStyle=(String)(event.getItem()).getLabel();
        //newFont(currentFontName,currentFontStyle,currentFontSize);
        //System.out.println(currentFont.toString());
      }
    }
    // Size Menu
    else if(name=="fontSize"){
      if(event.getStateChange()==ItemEvent.SELECTED){
        currentFontSize=defaultSettings.translateFontSize(((CheckboxMenuItem)source).getLabel());
      }
      newFont(currentFontName,currentFontStyle,currentFontSize);
      //System.out.println(currentFont.toString());
    }
    else if(name=="Tools"){
      String text="";
      if(((CheckboxMenuItem)source).getLabel()=="Text"){
        text="TEXT MODE";
        mode=GraphicalObject.TEXT;
      }
      else if(((CheckboxMenuItem)source).getLabel()=="Line"){
        text="LINE MODE";
        mode=GraphicalObject.LINE;
      }
      else if(((CheckboxMenuItem)source).getLabel()=="Oval"){
        text="OVAL MODE";
        mode=GraphicalObject.OVAL;
      }
      else if(((CheckboxMenuItem)source).getLabel()=="Rectangle"){
        text="RECTANGLE MODE";
        mode=GraphicalObject.RECTANGLE;
      }
      else if(((CheckboxMenuItem)source).getLabel()=="Filled Oval"){
        text="SOLID OVAL MODE";
        mode=GraphicalObject.FILLED_OVAL;
      }
      else if(((CheckboxMenuItem)source).getLabel()=="Filled Rectangle"){
        text="SOLID RECTANGLE MODE";
        mode=GraphicalObject.FILLED_RECTANGLE;
      }
      else if(((CheckboxMenuItem)source).getLabel()=="Freehand"){
        text="FREHAND MODE";
        mode=GraphicalObject.FREEHAND;
      }
      gui.writeToStatusBar(true,text,true);

    }

    // Font Name Choice
    else if(name=="fontNames"){
      //System.out.println(" kuken paa ernames");
      if(event.getStateChange()==ItemEvent.SELECTED){
        currentFontName=(String)event.getItem();
        newFont(currentFontName,currentFontStyle,currentFontSize);
        //System.out.println(currentFont.toString());
      }
    }
    // Font Size Choice
    else if(name=="fontSizes"){
      //System.out.println("sizes");
      if(event.getStateChange()==ItemEvent.SELECTED){
        currentFontSize=defaultSettings.translateFontSize(((Choice)source).getSelectedItem());
        newFont(currentFontName,currentFontStyle,currentFontSize);
        //System.out.println(currentFont.toString());
      }
    }
    // Line Width Choice
    else if(name=="lineWidths"){
      if(event.getStateChange()==ItemEvent.SELECTED){
        currentLineWidth=defaultSettings.translateLineWidth(((Choice)source).getSelectedIndex());
        //System.out.println(currentLineWidth);
      }
    }
  }

  void testing(){
    gui.writeToStatusBar(true,"kuken paa er !",true);
  }

  /**
   *  Listen to Mouse Events generated in GUI.java
   *
   *  @param  event   A MouseEvent object with the
   *                  specified source component, type,
   *                  modifiers, coordinates, and click count.
   *
   */
  public void mousePressed(MouseEvent event){
    Object source=event.getComponent();
    // Color Palette
    if(source instanceof ColorPalette){
      if(event.getModifiers()==MouseEvent.BUTTON3_MASK){
        setBackgroundColor(((ColorPalette)source).getBackgroundColor());
        owner.sendBackgroundColorData(getBackgroundColor());
        gui.showBackgroundColor(getBackgroundColor());
      }
      else{
        setColor(((ColorPalette)source).getForegroundColor());
        gui.showForegroundColor(getColor());
        if(selected){
          for(int i=0;i<objects.size();i++){
            if(((GraphicalObject)objects.elementAt(i)).getId()==selectedId){
              ((GraphicalObject)objects.elementAt(i)).setColor(getColor());
              sendChangeObject(selectedId,0,0);//(GraphicalObject)objects.elementAt(i));
            }
          }
          repaint(boundingBox.getX(),boundingBox.getY(),boundingBox.getW(),boundingBox.getH());
        }
      }
    }
    // Image Buttons
    else if(source instanceof ImageButton){
      String name=(event.getComponent()).getName();
      String text="";
      if(cursor!=null) repaint(cursor.getX(),cursor.getY(),cursor.getW(),cursor.getH());
      cursor=null;
      if(selected/*boundingBox!=null*/) repaint(boundingBox.getX(),boundingBox.getY(),boundingBox.getW(),boundingBox.getH());
      selected=false;

      if(name=="Print"){
        print();
      }

      else if(name=="Bold Font"){
        setBold();
      }

      else if(name=="Italic Font"){
        setItalic();
      }


      else if(name=="Pointer Tool"){
        text=new String("SELECT MODE");
        mode=GraphicalObject.SELECT;

      }

      else if(name=="Text Tool"){
        text=new String("TEXT MODE");
        mode=GraphicalObject.TEXT;
      }

      else if(name=="Line Tool"){
        text=new String("LINE MODE");
        mode=GraphicalObject.LINE;
      }

      else if(name=="Oval Tool"){
        text=new String("OVAL MODE");
        mode=GraphicalObject.OVAL;
      }

      else if(name=="Rectangle Tool"){
        text=new String("RECTANGLE MODE");
        mode=GraphicalObject.RECTANGLE;
      }

      else if(name=="Solid Oval Tool"){
          text=new String("FILLED OVAL MODE");
          mode=GraphicalObject.FILLED_OVAL;
      }

      else if(name=="Solid Rectangle Tool"){
          text=new String("FILLED RECTANGLE MODE");
          mode=GraphicalObject.FILLED_RECTANGLE;
      }

      else if(name=="Freehand Tool"){
          text=new String("FREEHAND MODE");
          mode=GraphicalObject.FREEHAND;
      }
      gui.writeToStatusBar(true,text,true);

    }


  }
  public void mouseEntered(MouseEvent event){
  }

  public void mouseExited(MouseEvent event){
  }

  public void mouseClicked(MouseEvent event){
  }

  public void mouseReleased(MouseEvent event){
  }

  /**
   *  Listen to Action Events
   *
   *  @param  event   An ActionEvent object with
   *                  the specified source object.
   *
   */
  public void actionPerformed(ActionEvent event){
    String cmd=event.getActionCommand();

    // file Menu
    if(cmd.equals("print")){
      print();
    }

    // Edit Menu
    else if(cmd.equals("delete")){
      if(deleteObject(selectedId)){
        selected=false;
      }
      owner.sendDeleteData(selectedId);
    }

    else if(cmd.equals("clear")){
      clear();
    }

  }

  // Colors
  private Color foregroundColor;
  private Color backgroundColor;

  // Classes
  private GUI gui;
  private DefaultSettings defaultSettings=new DefaultSettings();

  //Mode
  private int mode;

  // Font
  private FontMetrics fontMetrics;
  private Font currentFont;
  private String currentFontName;
  private int currentFontSize;
  private int currentFontStyle;

  // Line Width
  private int currentLineWidth;

  private GraphicalObject current=null;
  private GraphicalObject remoteObject=null;
  private GraphicalObject point=new GraphicalObject();
  private GraphicalObject textObject=null;
  private GraphicalObject remoteTextObject=null;
  private ResultProcessor owner;
  private Vector objects;
  private TextCursor cursor;


  private final static int INITIAL_SIZE=100;
  private final static int INCREMENT=100;

  private int freehandId,firstElement,/*lastElement,*/
              newX,newY,initialX,initialY,tempId,index,stringLength,mouseX,mouseY;

  private boolean newObject,debug=false,first,position,newLine,safeMode=true, draged=false;

  private int defaultObjectId;
  private int currentObjectId;

  private BoundingBox boundingBox;

  boolean selected=false;
  int selectedId;



}

//////////////////////////////////////////////////////////

class TextCursor{
  int baseLineX,baseLineY,ascent,width;
  Color color;

  public TextCursor(int x,int y,int w,int h,Color c){
    baseLineX=x;
    baseLineY=y;
    ascent=h;
    width=w;
    color=c;
  }

  public int getX(){
    return baseLineX;
  }

  public int getY(){
    return baseLineY-ascent;
  }

  public int getW(){
    return width;
  }

  public int getH(){
    return ascent;
  }

  public void draw(Graphics g){
    g.setXORMode(color);
    g.fillRect(baseLineX,baseLineY-ascent,width,ascent);
    g.setPaintMode();
  }
}


class BoundingBox{

  private int x1,y1,x2,y2,x,y,w,h,s;
  private Color color;
  private GraphicalObject obj;
  private Vector objects;


  public BoundingBox(int id,Vector objects,Color color){
    this.color=color;
    this.objects=objects;
    x1= (new Integer(Integer.MAX_VALUE)).intValue();
    y1= (new Integer(Integer.MAX_VALUE)).intValue();
    x2= (new Integer(Integer.MIN_VALUE)).intValue();
    y2= (new Integer(Integer.MIN_VALUE)).intValue();
    obj=(GraphicalObject)objects.elementAt(findFirstElement(id));
    s=8; // corner size
    createBoundingBox(id);

  }

  void createBoundingBox(int id){
    if(obj.getType()==GraphicalObject.TEXT){
      for(int i=0;i<objects.size();i++){
        if(((GraphicalObject)objects.elementAt(i)).getId()==id){

          if(((GraphicalObject)objects.elementAt(i)).startX<x1) x1=((GraphicalObject)objects.elementAt(i)).startX;

          if(((GraphicalObject)objects.elementAt(i)).startY<y1)
            y1=((GraphicalObject)objects.elementAt(i)).startY-((GraphicalObject)objects.elementAt(i)).endY;

          if(((GraphicalObject)objects.elementAt(i)).startX+((GraphicalObject)objects.elementAt(i)).endX>x2)
            x2=((GraphicalObject)objects.elementAt(i)).startX+((GraphicalObject)objects.elementAt(i)).endX;

          if(((GraphicalObject)objects.elementAt(i)).startY>y2)
            y2=((GraphicalObject)objects.elementAt(i)).startY;
        }
      }
    }
    else if(obj.getType()==GraphicalObject.FREEHAND){
      int width=obj.getLineWidth();
      for(int i=0;i<objects.size();i++){
        if(((GraphicalObject)objects.elementAt(i)).getId()==id){

          if(((GraphicalObject)objects.elementAt(i)).startX<x1)
            x1=((GraphicalObject)objects.elementAt(i)).startX;

          if(((GraphicalObject)objects.elementAt(i)).startY<y1)
            y1=((GraphicalObject)objects.elementAt(i)).startY;

          if(((GraphicalObject)objects.elementAt(i)).startX>x2)
            x2=((GraphicalObject)objects.elementAt(i)).startX+width;

          if(((GraphicalObject)objects.elementAt(i)).startY>y2)
            y2=((GraphicalObject)objects.elementAt(i)).startY+width;
        }
      }
    }
    else{
      x1=Math.min(obj.startX,obj.endX);
      y1=Math.min(obj.startY,obj.endY);
      x2=Math.max(obj.startX,obj.endX);
      y2=Math.max(obj.startY,obj.endY);
    }
    x=x1-2;
    y=y1-2;
    w=x2-x1+4;
    h=y2-y1+4;
  }

  /**
   * Find the first vector element whith the specific Id.
   *
   * @param   id      Id of the GraphicalObject
   * @return  pos     Return the vector position of the element
   *                  described by id.
   */
  int findFirstElement(int id){
    for(int i=0;i<objects.size();i++){
      if(id==((GraphicalObject)objects.elementAt(i)).getId()) return i;
    }
    return 0;
  }

  public int getX(){
    return x-s;
  }

  public int getY(){
    return y-s;
  }

  public int getW(){
    return x+w+s;
  }

  public int getH(){
    return y+h+s;
  }

  public boolean isInside(int xPos, int yPos){
    if(xPos>x && yPos>y && xPos<x+w && yPos<y+h) return true;
    else return false;
  }

  public void draw(Graphics g){
    g.setXORMode(color);

    g.fillRect(x-s,y-s,s,s);
    g.fillRect(x+w,y-s,s,s);
    g.fillRect(x-s,y+h,s,s);
    g.fillRect(x+w,y+h,s,s);

    g.setPaintMode();
  }
}

