/**
 *  Class GUI.java
 *  @author: Anders Sandstrom
 *  UCCS 1997
 */

// Import
import java.awt.*;
import java.awt.event.*;
import imgbtn.*;
import extended.*;
import colortools.*;

public class GUI extends Frame implements ItemListener{
  public GUI(Core core){
    super("Zodega");
    this.core=core;
    addWindowListener(core);
    setup();
    setSize(defaultSettings.getWindowWidth(),defaultSettings.getWindowHeight());
    pack();
    show();
    init();
  }

  /**
   *  Setup GUI.
   *
   */
  void setup(){
    setLayout(new BorderLayout());
    setBackground(SystemColor.window);
    setupFonts();
    setupCanvas();
    setupMenus();
    setupMenuBar();
    setMenuBar(menuBar);
    setupButtonBars();
    setupToolBar();
    setupStatusBar();
  }

  /**
   *  Listen to Item Events generated in GUI.java
   *
   *  @param  event   An instance of item event
   *                  emitted by ItemSelectable objects.
   *                  This event is generated when an
   *                  item is selected or de-selected
   *
   */
  public void itemStateChanged(ItemEvent event){
    String name;
    Object source=event.getSource();
    if(source instanceof Choice){
      name=((Choice)source).getName();
    }
    else {
      name=((CheckboxMenuItem)source).getName();
    }

    // Font Menu Items
    if(name=="fontName"){
      for(int i=0;i<fontMenu.getItemCount();i++){
        if(((CheckboxMenuItem)source).getLabel()==(fontMenu.getItem(i)).getLabel()){
          ((CheckboxMenuItem)fontMenu.getItem(i)).setState(true);
          fontNameChoice.select((fontMenu.getItem(i)).getLabel());
        }
        else{
          ((CheckboxMenuItem)fontMenu.getItem(i)).setState(false);
        }
      }
    }
    // Size Menu Items
    else if(name=="fontSize"){
      for(int i=0;i<sizeMenu.getItemCount();i++){
        if(((CheckboxMenuItem)source).getLabel()==(sizeMenu.getItem(i)).getLabel()){
          ((CheckboxMenuItem)sizeMenu.getItem(i)).setState(true);
          fontSizeChoice.select((sizeMenu.getItem(i)).getLabel());
        }
        else ((CheckboxMenuItem)sizeMenu.getItem(i)).setState(false);
      }
    }
    // Style Menu items
    else if(name=="fontStyle"){
      for(int i=0;i<styleMenu.getItemCount();i++){
        if(((CheckboxMenuItem)source).getLabel()==(styleMenu.getItem(i)).getLabel())
          ((CheckboxMenuItem)styleMenu.getItem(i)).setState(true);
        else ((CheckboxMenuItem)styleMenu.getItem(i)).setState(false);
      }
    }
    // Font Menu Items
    else if(name=="Tools"){
      for(int i=0;i<toolsMenu.getItemCount();i++){
        if(((CheckboxMenuItem)source).getLabel()==(toolsMenu.getItem(i)).getLabel()){
          ((CheckboxMenuItem)toolsMenu.getItem(i)).setState(true);
        }
        else{
          ((CheckboxMenuItem)toolsMenu.getItem(i)).setState(false);
        }
      }
    }


    // Font Name Choice
    else if(name=="FontNames"){
    }
    // Font Style Choice
    else if(name=="FontSizes"){
    }

  }



  /**
   *  Writes text in the right or left feild of status bar.
   *
   *  @param  b     Boolean LEFT for left text field and boolean
   *                boolean RIGHT for right text field.
   *  @param  str   Text string.
   *
   */
  public void writeToStatusBar(boolean b,String str, boolean memory){
    if(memory) toolStatus=str;
    if(b) leftInfoField.setText(str);
    else rightInfoField.setText(str);
  }


  public void undoWriteToStatusBar(boolean b){
    if(b) leftInfoField.setText(toolStatus);
    else rightInfoField.setText(toolStatus);
  }


  public String getToolStatus(){
    return toolStatus;
  }

  public void setToolStatus(String string){
    toolStatus=string;
  }


  public void showForegroundColor(Color color){
    currentColors.setForegroundColor(color);
  }

  public void showBackgroundColor(Color color){
    currentColors.setBackgroundColor(color);
  }

  public String[] getFontNames(){
    return fontNames;
  }


  /**
   *  Sets up menus.
   *
   */
  void setupMenus(){

    // File Menu
    fileMenu= new Menu("File");
    fileMenu.add(new MenuItem("New", new MenuShortcut('N')));
    ((MenuItem)fileMenu.getItem(0)).addActionListener(core);
    ((MenuItem)fileMenu.getItem(0)).setActionCommand("new");


    fileMenu.add(new MenuItem("Open", new MenuShortcut('O')));
    ((MenuItem)fileMenu.getItem(1)).addActionListener(core);
    ((MenuItem)fileMenu.getItem(1)).setActionCommand("open");

    fileMenu.add(new MenuItem("Save", new MenuShortcut('S')));
    ((MenuItem)fileMenu.getItem(2)).addActionListener(core);
    ((MenuItem)fileMenu.getItem(2)).setActionCommand("save");

    fileMenu.add(new MenuItem("Save As...", new MenuShortcut('A')));
    ((MenuItem)fileMenu.getItem(3)).addActionListener(core);
    ((MenuItem)fileMenu.getItem(3)).setActionCommand("saveas");

    fileMenu.addSeparator();

    fileMenu.add(new MenuItem("Connect...", new MenuShortcut('C')));
    ((MenuItem)fileMenu.getItem(5)).addActionListener(core);
    ((MenuItem)fileMenu.getItem(5)).setActionCommand("connect");

    fileMenu.add(new MenuItem("Disconnect", new MenuShortcut('D')));
    ((MenuItem)fileMenu.getItem(6)).addActionListener(core);
    ((MenuItem)fileMenu.getItem(6)).setActionCommand("disconnect");

    fileMenu.add(new MenuItem("Conference Info...", new MenuShortcut('I')));
    ((MenuItem)fileMenu.getItem(7)).addActionListener(core);
    ((MenuItem)fileMenu.getItem(7)).setActionCommand("info");

    fileMenu.addSeparator();

    fileMenu.add(new MenuItem("Print", new MenuShortcut('P')));
    ((MenuItem)fileMenu.getItem(9)).addActionListener(canvas);
    ((MenuItem)fileMenu.getItem(9)).setActionCommand("print");

    fileMenu.addSeparator();

    fileMenu.add(new MenuItem("Exit"));
    ((MenuItem)fileMenu.getItem(11)).addActionListener(core);
    ((MenuItem)fileMenu.getItem(11)).setActionCommand("exit");


    // Edit Menu
    editMenu= new Menu("Edit");
    editMenu.add(new MenuItem("Delete"));
    ((MenuItem)editMenu.getItem(0)).addActionListener(canvas);
    ((MenuItem)editMenu.getItem(0)).setActionCommand("delete");

    editMenu.add(new MenuItem("Clear"));
    ((MenuItem)editMenu.getItem(1)).addActionListener(canvas);
    ((MenuItem)editMenu.getItem(1)).setActionCommand("clear");

    // Text Menu
    textMenu= new Menu("Text");
    // Font Submenu
    fontMenu=new Menu("Font");
    for(int i=0;i<fontNames.length;i++){
      fontMenu.add(new CheckboxMenuItem(fontNames[i]));
      ((CheckboxMenuItem)fontMenu.getItem(i)).setName("fontName");
      ((CheckboxMenuItem)fontMenu.getItem(i)).addItemListener(this);
      ((CheckboxMenuItem)fontMenu.getItem(i)).addItemListener(canvas);
    }

    // Style Submenu
    styleMenu=new Menu("Style");
    for(int i=0;i<fontStyles.length;i++){
      styleMenu.add(new CheckboxMenuItem(fontStyles[i]));
      ((CheckboxMenuItem)styleMenu.getItem(i)).setName("fontStyle");
      ((CheckboxMenuItem)styleMenu.getItem(i)).addItemListener(this);
      ((CheckboxMenuItem)styleMenu.getItem(i)).addItemListener(canvas);

    }
    // Size Submenu
    sizeMenu=new Menu("Size");
    for(int i=0;i<fontSizes.length;i++){
      sizeMenu.add(new CheckboxMenuItem(fontSizes[i]));
      ((CheckboxMenuItem)sizeMenu.getItem(i)).setName("fontSize");
      ((CheckboxMenuItem)sizeMenu.getItem(i)).addItemListener(this);
      ((CheckboxMenuItem)sizeMenu.getItem(i)).addItemListener(canvas);
    }
    textMenu.add(fontMenu);
    textMenu.add(styleMenu);
    textMenu.add(sizeMenu);

    // Tools Menu
    toolsMenu= new Menu("Tools");
    toolsMenu.add(new CheckboxMenuItem("Text"));
    toolsMenu.add(new CheckboxMenuItem("Line"));
    toolsMenu.add(new CheckboxMenuItem("Oval"));
    toolsMenu.add(new CheckboxMenuItem("Filled Oval"));
    toolsMenu.add(new CheckboxMenuItem("Rectangle"));
    toolsMenu.add(new CheckboxMenuItem("Filled Rectangle"));
    toolsMenu.add(new CheckboxMenuItem("Freehand"));
    for(int i=0;i<toolsMenu.getItemCount();i++){
      (toolsMenu.getItem(i)).setName("Tools");
      ((CheckboxMenuItem)toolsMenu.getItem(i)).addItemListener(this);
      ((CheckboxMenuItem)toolsMenu.getItem(i)).addItemListener(canvas);
    }


    // Help Menu
    helpMenu= new Menu("Help");
    helpMenu.add(new MenuItem("About..."));
    ((MenuItem)helpMenu.getItem(0)).addActionListener(core);
    ((MenuItem)helpMenu.getItem(0)).setActionCommand("about");


  }
  /**
   *  Setup menubar.
   *
   */
  void setupMenuBar(){
    menuBar= new MenuBar();
    menuBar.add(fileMenu);
    menuBar.add(editMenu);
    menuBar.add(textMenu);
    menuBar.add(toolsMenu);
    menuBar.add(helpMenu);
  }

  /**
   *  Setup fonts.
   *
   */
  void setupFonts(){
    toolkit=getToolkit();
    fontNames=toolkit.getFontList();
    int length=(defaultSettings.getFontStyles()).length;
    fontStyles=new String[length];
    for(int i=0;i<length;i++){
      fontStyles[i]=(defaultSettings.getFontStyles())[i].toString();
    }
    length=(defaultSettings.getFontSizes()).length;
    fontSizes=new String[length];
    for(int i=0;i<length;i++){
      fontSizes[i]=(defaultSettings.getFontSizes())[i].toString();
    }

  }

  /**
   * Method setupCanvas()
   * Adds a canvas in the middle of the JavaDraw window.
   */
  void setupCanvas(){
    canvas=new ExtendedCanvas(this,core,defaultSettings.getMode());
    //canvas.addMouseListener(core);
    add("Center",canvas);
  }

  /**
   * Method setupButtonBars()
   * Adds two buttonbars at the top of the JavaDraw window.
   */
  void setupButtonBars(){
    Panel northPanel =new Panel();
    Panel buttonPanel1= new Panel();
    Panel buttonPanel2= new Panel();
    northPanel.setLayout(new GridLayout(2,1));
    buttonPanel1.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel2.setLayout(new FlowLayout(FlowLayout.LEFT));

    northPanel.setBackground(SystemColor.control);
    buttonPanel1.setBackground(SystemColor.control);
    buttonPanel2.setBackground(SystemColor.control);

    Image image;

    // New Button
    newImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/new1.gif"),"New");
    newImageButton.addMouseListener(core);
    //newImageButton.addMouseListener(this);
    newImageButton.setName("New");
    buttonPanel1.add(newImageButton);

    // Open Button
    openImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/open1.gif"),"Open");
    openImageButton.addMouseListener(core);
    //openImageButton.addMouseListener(this);
    openImageButton.setName("Open");
    buttonPanel1.add(openImageButton);

    // Save Button
    saveImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/save1.gif"),"Save");
    saveImageButton.addMouseListener(core);
    //saveImageButton.addMouseListener(this);
    saveImageButton.setName("Save");
    buttonPanel1.add(saveImageButton);

    // Print Button
    printImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/print1.gif"),"kuk");
    printImageButton.addMouseListener(canvas);
    printImageButton.addMouseListener(core);
    printImageButton.setName("Print");
    buttonPanel1.add(printImageButton);

    // Space
    buttonPanel1.add(new Label(" "));

    // Chatting Button
    chatImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/chat1.gif"),"Chatting");
    chatImageButton.addMouseListener(core);
    //chatImageButton.addMouseListener(this);
    chatImageButton.setName("Chatting");
    buttonPanel1.add(chatImageButton);

    // Space
    buttonPanel1.add(new Label(" "));

    // Connect Button
    connectImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/connect2.gif"),"Connect");
    connectImageButton.addMouseListener(core);
    //connectImageButton.addMouseListener(this);
    connectImageButton.setName("Connect");
    buttonPanel1.add(connectImageButton);

    // Disconnect Button
    disconnectImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/disconnect1.gif"),"Disconnect");
    disconnectImageButton.addMouseListener(core);
    //disconnectImageButton.addMouseListener(this);
    disconnectImageButton.setName("Disconnect");
    buttonPanel1.add(disconnectImageButton);

    // Font Name Selection
    fontNameChoice =new ExtendedChoice(fontNames);
    fontNameChoice.setName("fontNames");
    fontNameChoice.addItemListener(canvas);
    buttonPanel2.add(fontNameChoice);

    // Font Size Selection
    fontSizeChoice=new ExtendedChoice(fontSizes);
    fontSizeChoice.setName("fontSizes");
    fontSizeChoice.addItemListener(canvas);
    buttonPanel2.add(fontSizeChoice);

    // Bold Button
    boldImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/bold1.gif"),"Bold Font");
    boldImageButton.addMouseListener(core);
    boldImageButton.addMouseListener(canvas);
    boldImageButton.setName("Bold Font");
    buttonPanel2.add(boldImageButton);

    // Italic Button
    italicImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/italic1.gif"),"Italic Font");
    italicImageButton.addMouseListener(core);
    italicImageButton.addMouseListener(canvas);
    italicImageButton.setName("Italic Font");
    buttonPanel2.add(italicImageButton);

    // Line Width Label
    buttonPanel2.add(new Label("    Line width"));

    // Line Width Selection
    int length=(defaultSettings.getLineWidths()).length;
    lineWidths=new String[length];
    for(int i=0;i<length;i++){
      lineWidths[i]=(defaultSettings.getLineWidths())[i].toString();
    }
    lineWidthChoice=new ExtendedChoice(lineWidths);
    lineWidthChoice.setName("lineWidths");
    lineWidthChoice.addItemListener(canvas);
    buttonPanel2.add(lineWidthChoice);

    northPanel.add(buttonPanel1);
    northPanel.add(buttonPanel2);
    add("North",northPanel);


  }

  /**
   * Method setupToolBar()
   * Adds a toolbar at the left side of the window.
   */
  void setupToolBar(){

    Image image;

    Panel toolPanel =new Panel();

    Panel bPanel1= new Panel();
    Panel bPanel2= new Panel();
    Panel bPanel3= new Panel();

    toolPanel.setLayout(new BorderLayout());
    toolPanel.setBackground(SystemColor.control);

    bPanel1.setLayout(new GridLayout(4,2));
    bPanel1.setBackground(SystemColor.control);

    bPanel2.setLayout(new FlowLayout(FlowLayout.LEFT));
    bPanel2.setBackground(SystemColor.control);

    bPanel3.setLayout(new FlowLayout(FlowLayout.LEFT));
    bPanel3.setBackground(SystemColor.control);

    // Pointer Button
    pointerImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/pointer1.gif"),"Pointer" );
    pointerImageButton.addMouseListener(canvas);
    pointerImageButton.addMouseListener(core);
    pointerImageButton.setName("Pointer Tool");
    bPanel1.add( pointerImageButton );

    // Text Button
    textImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/text1.gif"),"Text Tool" );
    textImageButton.addMouseListener(canvas);
    textImageButton.addMouseListener(core);
    textImageButton.setName("Text Tool");
    bPanel1.add( textImageButton );

    // Line Button
    lineImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/line1.gif"),"Line Tool" );
    lineImageButton.addMouseListener(canvas);
    lineImageButton.addMouseListener(core);
    lineImageButton.setName("Line Tool");
    bPanel1.add( lineImageButton );

    // Oval Button
    ovalImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/oval1.gif"),"Oval Tool");
    ovalImageButton.addMouseListener(canvas);
    ovalImageButton.addMouseListener(core);
    ovalImageButton.setName("Oval Tool");
    bPanel1.add( ovalImageButton );

    // Rectangle Button
    rectangleImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/rectangle1.gif"),"Rectangle Tool" );
    rectangleImageButton.addMouseListener(canvas);
    rectangleImageButton.addMouseListener(core);
    rectangleImageButton.setName("Rectangle Tool");
    bPanel1.add( rectangleImageButton );

    // Filled Oval Button
    filledOvalImageButton =new ImageButton(image=toolkit.getImage("./imgbtn/oval2.gif"),"Solid Oval Tool");
    filledOvalImageButton.addMouseListener(canvas);
    filledOvalImageButton.addMouseListener(core);
    filledOvalImageButton.setName("Solid Oval Tool");
    bPanel1.add( filledOvalImageButton );

    // Filled Rectangle Button
    filledRectangleImageButton =new ImageButton(image=toolkit.getImage("./imgbtn/rectangle2.gif"),"Solid Rectangle Tool");
    filledRectangleImageButton.addMouseListener(canvas);
    filledRectangleImageButton.addMouseListener(core);
    filledRectangleImageButton.setName("Solid Rectangle Tool");
    bPanel1.add( filledRectangleImageButton );

    // Freehand Button
    freehandImageButton =new ImageButton(image=toolkit.getImage("./imgbtn/freehand1.gif"),"Freehand Tool");
    freehandImageButton.addMouseListener(canvas);
    freehandImageButton.addMouseListener(core);
    freehandImageButton.setName("Freehand Tool");
    bPanel1.add( freehandImageButton );

    // Color Palette
    colorPalette=new ColorPalette(3,"Color Palette");
    colorPalette.addMouseListener(core);
    colorPalette.addMouseListener(canvas);
    colorPalette.setName("Color Palette");
    bPanel2.add(colorPalette);

    // Current Color Indicator
    bPanel3.add(currentColors=new CurrentColors(canvas.getColor(),canvas.getBackgroundColor()));


    toolPanel.add("North",bPanel1);
    toolPanel.add("West",bPanel2);
    toolPanel.add("South",bPanel3);
    add("West",toolPanel);




  }


  /**
   * Method setupStatusBar()
   * Adds a statusbar at the botom of the JavaDraw window.
   */
  void setupStatusBar(){
    Panel statusBar = new Panel();
    statusBar.setLayout(new FlowLayout(FlowLayout.LEFT));
    statusBar.setBackground(SystemColor.control);

    leftInfoField = new TextField(35);
    leftInfoField.setEditable(false);
    rightInfoField = new TextField(35);
    rightInfoField.setEditable(false);

    statusBar.add(leftInfoField);
    statusBar.add(rightInfoField);
    add("South", statusBar);
  }



  /**
   *  Initialize variables.
   *
   */
  void init(){
    toolStatus="";
    disconnectImageButton.setEnabled(false);
    chatImageButton.setEnabled(false);
    fileMenu.getItem(6).setEnabled(false);

    String str="";
    fontNameChoice.select(defaultSettings.getFontName(fontNames));
    fontSizeChoice.select(str.valueOf(defaultSettings.getFontSize()));
    lineWidthChoice.select(str.valueOf(defaultSettings.getLineWidth()));

    for(int i=0;i<fontMenu.getItemCount();i++){
      if((fontMenu.getItem(i)).getLabel()==defaultSettings.getFontName(fontNames)){
        ((CheckboxMenuItem)fontMenu.getItem(i)).setState(true);
      }
    }

    for(int i=0;i<sizeMenu.getItemCount();i++){
      if((sizeMenu.getItem(i)).getLabel()==str.valueOf(defaultSettings.getFontSize())){
        ((CheckboxMenuItem)sizeMenu.getItem(i)).setState(true);
      }
    }


  }

  // Variable Declarations

  private static final boolean LEFT=true;
  private static final boolean RIGHT=false;

  // Menus
  public  Menu fileMenu;
  private Menu editMenu;
  private Menu textMenu;
  private Menu fontMenu;
  private Menu sizeMenu;
  private Menu styleMenu;
  private Menu toolsMenu;
  private Menu helpMenu;

  // Menu Bar
  private MenuBar menuBar;

  // Toolkit
  public Toolkit toolkit;

  // Strings
  private String fontNames[];
  private String fontStyles[];
  private String fontSizes[];

  private String lineWidths[];

  private String toolStatus;

  // Text Fields
  public TextField leftInfoField;// old msgtxt
  private TextField rightInfoField;// old infotxt


  // Image Buttons
  private ImageButton newImageButton;
  private ImageButton openImageButton;
  private ImageButton saveImageButton;
  private ImageButton printImageButton;

  public ImageButton chatImageButton;

  public ImageButton connectImageButton;
  public ImageButton disconnectImageButton;

  private ImageButton boldImageButton;
  private ImageButton italicImageButton;

  private ImageButton pointerImageButton;
  private ImageButton textImageButton;
  private ImageButton lineImageButton;
  private ImageButton ovalImageButton;
  private ImageButton rectangleImageButton;
  private ImageButton filledOvalImageButton;
  private ImageButton filledRectangleImageButton;
  private ImageButton freehandImageButton;

  // Choices
  private ExtendedChoice fontNameChoice;
  private ExtendedChoice fontSizeChoice;
  private ExtendedChoice lineWidthChoice;

  // Extended Canvas
  public ExtendedCanvas canvas;

  // Core
  private Core core;

  // Current Color Indicator
  private CurrentColors currentColors;

  // Color Palette
  private ColorPalette colorPalette;

  // Default Settings
  private DefaultSettings defaultSettings=new DefaultSettings();





}


