/**
 *  Class InfoDialog.java
 *  @author: Anders Sandstrom
 *  UCCS 1997
 */
import java.awt.event.*;
import java.awt.*;
import java.util.*;

class InfoDialog extends Dialog implements ItemListener{


  public InfoDialog(Core core,Frame gui){
    super(gui, "List of Clients Connected to Server", true);
    this.core=core;
    setup();
  }

  public InfoList getList(){
    return infoList;
  }

  public void updateList(String[] strings, int maxIndex){
    infoList.updateList(strings, maxIndex);
  }

  public void clearList(){
    revokeTextField.setText("");
    timeTextField.setText("");
    infoList.removeAll();
  }

  public synchronized void setVisible(boolean b) {
    Rectangle parentBounds = getParent().getBounds();
    Rectangle abounds = getBounds();
    setLocation(parentBounds.x + (parentBounds.width - abounds.width)/ 2,
    parentBounds.y + (parentBounds.height - abounds.height)/2);
    super.setVisible(b);
  }

  public String getUserName(){
    String name = revokeTextField.getText();
    if(name.length() != 0){
      StringTokenizer inputbufferST = new StringTokenizer(name, " ");
      return inputbufferST.nextToken();
    }
    else return "";
  }

  public String getTime(){
    return timeTextField.getText();
  }

  public void itemStateChanged(ItemEvent event){
    if(event.getStateChange()==ItemEvent.SELECTED){
      revokeTextField.setText(((List)event.getSource()).getSelectedItem());
    }
    else if(event.getStateChange()==ItemEvent.DESELECTED){
      revokeTextField.setText("");
    }
  }

  void setup(){

    setLayout(new BorderLayout());

    infoList=new InfoList(this);
    add("Center",infoList);

    Panel north = new Panel();

    revokeTextField = new TextField(20);
    revokeTextField.setEditable(false);
    north.add(revokeTextField);

    Label tmlbl = new Label("Revoke min:");
    north.add(tmlbl);

    timeTextField = new TextField(4);
    north.add(timeTextField);
    add("North", north);

    Panel south = new Panel();

    Button suspendButton = new Button("SUSPEND");
    suspendButton.addActionListener(core);
    suspendButton.setActionCommand("suspend");
    south.add(suspendButton);

    Button revokeButton = new Button("REVOKE");
    revokeButton.addActionListener(core);
    revokeButton.setActionCommand("revoke");
    south.add(revokeButton);

    Button cancelButton = new Button("CANCEL");
    cancelButton.addActionListener(core);
    cancelButton.setActionCommand("cancel");
    south.add(cancelButton);

    add("South", south);

    setSize(500,200);
    setBackground(Color.lightGray);
    pack();
    setResizable(false);
  }

  private Core core;
  private InfoList infoList;
  private TextField revokeTextField;
  private TextField timeTextField;
}

class InfoList extends List{

  public InfoList(InfoDialog infoDialog){
    super(4, false);
    addItemListener(infoDialog);
  }

  public void updateList(String[] str, int maxIndex){
    int index = 2;
    while(index < maxIndex){
      String item = padString(str[index++], " ", 25);
      item = item + str[index++];
      addItem(item);
    }
  }

  public String padString(String str, String padChar, int finalLength){
    int index = str.length();
    while(index < finalLength){
      str = str + padChar;
      index++;
    }
    return str;
  }
}