/**
 *  Class ColorPalette.java
 *  @author: Anders Sandstrom
 *  UCCS 1997
 */
package colortools;

import java.awt.*;
import java.lang.*;
import java.awt.event.*;

public class ColorPalette extends Canvas{

  private int defaultColorBoxWidth=20;
  private int defaultColorBoxHeight=20;
  private int defaultColumns=3;
  private int columns=defaultColumns;
  private int colorBoxWidth=defaultColorBoxWidth;
  private int colorBoxHeight=defaultColorBoxHeight;

  private String information="";

  private Color colors[]={Color.black,Color.blue,Color.cyan,Color.darkGray,Color.gray,
                  Color.green,Color.lightGray,Color.magenta,Color.orange,
                  Color.pink,Color.red,Color.white,Color.yellow};

  private Color paletteBackgroundColor=Color.lightGray;
  private Color defaultForegroundColor=Color.black;
  private Color currentForegroundColor=defaultForegroundColor;
  private Color defaultBackgroundColor=Color.white;
  private Color currentBackgroundColor=defaultBackgroundColor;

  public ColorPalette(){
    super();
    int width=colorBoxWidth*columns+1;
    int height=1+(int)Math.ceil((double)colors.length/(double)columns)*colorBoxHeight;
    setSize(width,height);
    setBackground(paletteBackgroundColor);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    setVisible(true);
  }

  public ColorPalette(String info){
    super();
    int width=colorBoxWidth*columns+1;
    int height=1+(int)Math.ceil((double)colors.length/(double)columns)*colorBoxHeight;
    information=info;
    setSize(width,height);
    setBackground(paletteBackgroundColor);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    setVisible(true);
  }


  public ColorPalette(int col){
    super();
    columns=col;
    int width=colorBoxWidth*columns+1;
    int height=1+(int)Math.ceil((double)colors.length/(double)columns)*colorBoxHeight;
    setSize(width,height);
    setBackground(paletteBackgroundColor);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    setVisible(true);
  }

  public ColorPalette(int col, String info){
    super();
    columns=col;
    int width=colorBoxWidth*columns+1;
    int height=1+(int)Math.ceil((double)colors.length/(double)columns)*colorBoxHeight;
    information=info;
    setSize(width,height);
    setBackground(paletteBackgroundColor);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    setVisible(true);
  }

  public ColorPalette(int x,int y,int col){
    super();
    columns=col;
    colorBoxWidth=x;
    colorBoxHeight=y;
    int width=colorBoxWidth*columns+1;
    int height=1+(int)Math.ceil((double)colors.length/(double)columns)*colorBoxHeight;
    setSize(width,height);
    setBackground(paletteBackgroundColor);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    setVisible(true);
  }

  public ColorPalette(int x,int y,int col,String info){
    super();
    columns=col;
    colorBoxWidth=x;
    colorBoxHeight=y;
    int width=colorBoxWidth*columns+1;
    int height=1+(int)Math.ceil((double)colors.length/(double)columns)*colorBoxHeight;
    information=info;
    setSize(width,height);
    setBackground(paletteBackgroundColor);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    setVisible(true);
  }

  public void paint(Graphics g){
    int x=0,y=0-colorBoxHeight;
    for(int i=0;i<colors.length;i++){
      g.setColor(colors[i]);
      if(i%columns==0){
        x=0;
        y+=colorBoxHeight;
      }
      g.fillRect(x+3,y+3,colorBoxWidth-6,colorBoxHeight-6);
      draw3dBorder(g,x,y,colorBoxWidth,colorBoxHeight);
      x+=colorBoxWidth;
    }
  }

  void setForegroundColor(int x, int y){
    int index=x/colorBoxWidth+y/colorBoxHeight*columns;
    if(index<colors.length)
      currentForegroundColor=colors[index];
  }

  public Color getForegroundColor(){
    return currentForegroundColor;
  }

  void setBackgroundColor(int x, int y){
    int index=x/colorBoxWidth+y/colorBoxHeight*columns;
    if(index<colors.length)
      currentBackgroundColor=colors[index];
  }
  public Color getBackgroundColor(){
    return currentBackgroundColor;
  }

  protected void processMouseEvent(MouseEvent event){
    if(event.getID()==event.MOUSE_PRESSED){
      Object source=event.getComponent();
      if(source==this && event.getModifiers()==MouseEvent.BUTTON3_MASK)
        setBackgroundColor(event.getX(),event.getY());
      else if(source==this)
        setForegroundColor(event.getX(),event.getY());
      super.processMouseEvent(event);
    }
  }

  void draw3dBorder(Graphics g,int x1,int y1,int x2, int y2){
    g.setColor(paletteBackgroundColor);
    g.drawRect(x1,y1,x2,y2);
    g.setColor(Color.white);
    g.drawLine(x1+1,y1+1,x1+x2-1,y1+1);
    g.drawLine(x1+1,y1+1,x1+1,y1+y2-2);
    g.drawLine(x1+3,y1+y2-3,x1+x2-3,y1+y2-3);
    g.drawLine(x1+x2-3,y1+3,x1+x2-3,y1+y2-3);
    g.setColor(Color.gray);
    g.drawLine(x1+1,y1+y2-1,x1+x2-1,y1+y2-1);
    g.drawLine(x1+x2-1,y1+2,x1+x2-1,y1+y2-1);
    g.drawLine(x1+3,y1+3,x1+x2-3,y1+3);
    g.drawLine(x1+3,y1+3,x1+3,y1+y2-4);
  }

/**
 * Deliver user defined ColorPalette information
 *
 * @return  ColorPalette string
 */
public String getMessage(){
  return information;
}


}


